@@ -1,6 +1,7 @@
 /// Decrypts asymmetrically-encrypted OpenPGP messages using the
 /// openpgp crate, Sequoia's low-level API.
 
+use std::sync::Arc;
 use std::collections::HashMap;
 use std::env;
 use std::io;
@@ -10,7 +11,6 @@ use anyhow::Context;
 use sequoia_openpgp as openpgp;
 
 use openpgp::*;
-use openpgp::cert::prelude::*;
 use openpgp::crypto::{KeyPair, SessionKey};
 use openpgp::types::SymmetricAlgorithm;
 use openpgp::parse::{
@@ -59,21 +59,23 @@ pub fn main() -> openpgp::Result<()> {
 /// keys for the signature verification and implements the
 /// verification policy.
 struct Helper {
-    keys: HashMap<KeyID, (Fingerprint, KeyPair)>,
+    keys: HashMap<KeyID, (Arc<Cert>, KeyPair)>,
 }
 
 impl Helper {
     /// Creates a Helper for the given Certs with appropriate secrets.
     fn new(p: &dyn Policy, certs: Vec<openpgp::Cert>) -> Self {
-        // Map (sub)KeyIDs to primary fingerprints and secrets.
+        // Map (sub)KeyIDs to certs and secrets.
         let mut keys = HashMap::new();
         for cert in certs {
+            let cert = Arc::new(cert);
+
             for ka in cert.keys().unencrypted_secret().with_policy(p, None)
                 .supported()
                 .for_storage_encryption().for_transport_encryption()
             {
                 keys.insert(ka.key().keyid(),
-                            (cert.fingerprint(),
+                            (cert.clone(),
                              ka.key().clone().into_keypair().unwrap()));
             }
         }
@@ -85,23 +87,22 @@ impl Helper {
 }
 
 impl DecryptionHelper for Helper {
-    fn decrypt<D>(&mut self,
-                  pkesks: &[openpgp::packet::PKESK],
-                  _skesks: &[openpgp::packet::SKESK],
-                  sym_algo: Option<SymmetricAlgorithm>,
-                  mut decrypt: D)
-                  -> openpgp::Result<Option<openpgp::Fingerprint>>
-        where D: FnMut(SymmetricAlgorithm, &SessionKey) -> bool
+    fn decrypt(&mut self,
+               pkesks: &[openpgp::packet::PKESK],
+               _skesks: &[openpgp::packet::SKESK],
+               sym_algo: Option<SymmetricAlgorithm>,
+               decrypt: &mut dyn FnMut(Option<SymmetricAlgorithm>, &SessionKey) -> bool)
+               -> openpgp::Result<Option<Cert>>
     {
         // Try each PKESK until we succeed.
         let mut recipient = None;
         for pkesk in pkesks {
-            if let Some((fp, pair)) = self.keys.get_mut(pkesk.recipient()) {
+            if let Some((cert, pair)) = self.keys.get_mut(&KeyID::from(pkesk.recipient())) {
                 if pkesk.decrypt(pair, sym_algo)
                     .map(|(algo, session_key)| decrypt(algo, &session_key))
                     .unwrap_or(false)
                 {
-                    recipient = Some(fp.clone());
+                    recipient = Some(cert.as_ref().clone());
                     break;
                 }
             }
