@@ -1,9 +1,9 @@
 //! ASCII Armor.
 //!
 //! This module deals with ASCII Armored data (see [Section 6 of RFC
-//! 4880]).
+//! 9580]).
 //!
-//!   [Section 6 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-6
+//!   [Section 6 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-6
 //!
 //! # Scope
 //!
@@ -46,6 +46,7 @@ use base64::Engine;
 use base64::engine::general_purpose::STANDARD as base64std;
 use base64::engine::general_purpose::STANDARD_NO_PAD as base64nopad;
 
+use crate::Profile;
 use crate::packet::prelude::*;
 use crate::packet::header::{BodyLength, CTBNew, CTBOld};
 use crate::parse::Cookie;
@@ -61,17 +62,19 @@ use crc::Crc;
 const TRACE: bool = false;
 
 /// The encoded output stream must be represented in lines of no more
-/// than 76 characters each (see (see [RFC 4880, section
-/// 6.3](https://tools.ietf.org/html/rfc4880#section-6.3).  GnuPG uses
-/// 64.
+/// than 76 characters each (see [Section 6 of RFC 9580]).  GnuPG
+/// uses 64.
+///
+/// [Section 6 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-6
 pub(crate) const LINE_LENGTH: usize = 64;
 
 const LINE_ENDING: &str = "\n";
 
-/// Specifies the type of data (see [RFC 4880, section 6.2]).
+/// Specifies the type of data (see [Section 6.2 of RFC 9580]).
 ///
-/// [RFC 4880, section 6.2]: https://tools.ietf.org/html/rfc4880#section-6.2
+/// [Section 6.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-6.2
 #[derive(Copy, Clone, Debug, PartialEq)]
+#[non_exhaustive]
 pub enum Kind {
     /// A generic OpenPGP message.  (Since its structure hasn't been
     /// validated, in this crate's terminology, this is just a
@@ -123,9 +126,9 @@ enum Label {
     Signature,
     /// A message using the Cleartext Signature Framework.
     ///
-    /// See [Section 7 of RFC 4880].
+    /// See [Section 7 of RFC 9580].
     ///
-    ///   [Section 7 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-7
+    ///   [Section 7 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-7
     CleartextSignature,
     /// A generic file.  This is a GnuPG extension.
     File,
@@ -242,6 +245,7 @@ impl Kind {
 /// A filter that applies ASCII Armor to the data written to it.
 pub struct Writer<W: Write> {
     sink: W,
+    profile: Option<Profile>,
     kind: Kind,
     stash: Vec<u8>,
     column: usize,
@@ -315,6 +319,7 @@ impl<W: Write> Writer<W> {
     {
         let mut w = Writer {
             sink: inner,
+            profile: None,
             kind,
             stash: Vec::<u8>::with_capacity(2),
             column: 0,
@@ -340,6 +345,48 @@ impl<W: Write> Writer<W> {
         Ok(w)
     }
 
+    /// Sets the version of OpenPGP to generate ASCII Armor for.
+    ///
+    /// This function can only be called once.  Calling it repeatedly
+    /// will return an error.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::io::{Read, Write, Cursor};
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::armor::{Writer, Kind};
+    ///
+    /// # fn main() -> std::io::Result<()> {
+    /// let mut writer = Writer::new(Vec::new(), Kind::File)?;
+    ///
+    /// writer.set_profile(openpgp::Profile::RFC9580)?;
+    /// writer.set_profile(openpgp::Profile::RFC9580).unwrap_err();
+    /// writer.set_profile(openpgp::Profile::RFC4880).unwrap_err();
+    ///
+    /// writer.write_all(b"Hello world!")?;
+    /// let buffer = writer.finalize()?;
+    /// assert_eq!(
+    ///     String::from_utf8_lossy(&buffer),
+    ///     "-----BEGIN PGP ARMORED FILE-----
+    ///
+    /// SGVsbG8gd29ybGQh
+    /// -----END PGP ARMORED FILE-----
+    /// ");
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn set_profile(&mut self, profile: Profile) -> Result<()> {
+        if self.profile.is_some() {
+            return Err(io::Error::new(
+                io::ErrorKind::Other,
+                "profile already selected"));
+        }
+
+        self.profile = Some(profile);
+        Ok(())
+    }
+
     /// Returns a reference to the inner writer.
     pub fn get_ref(&self) -> &W {
         &self.sink
@@ -403,14 +450,22 @@ impl<W: Write> Writer<W> {
             write!(self.sink, "{}", LINE_ENDING)?;
         }
 
-        // 24-bit CRC
-        let crc = self.crc.finalize();
-        let bytes = &crc.to_be_bytes()[1..4];
+        match self.profile {
+            None | Some(Profile::RFC4880) => {
+                // 24-bit CRC
+                let crc = self.crc.finalize();
+                let bytes = &crc.to_be_bytes()[1..4];
 
-        // CRC and footer.
-        write!(self.sink, "={}{}{}{}",
-               base64nopad.encode(&bytes),
-               LINE_ENDING, self.kind.end(), LINE_ENDING)?;
+                // Emit the CRC line.
+                write!(self.sink, "={}{}",
+                       base64nopad.encode(&bytes), LINE_ENDING)?;
+            },
+
+            Some(Profile::RFC9580) => (),
+        }
+
+        // Finally, emit the footer.
+        write!(self.sink, "{}{}", self.kind.end(), LINE_ENDING)?;
 
         self.dirty = false;
         crate::vec_truncate(&mut self.scratch, 0);
@@ -433,8 +488,14 @@ impl<W: Write> Write for Writer<W> {
         self.finalize_headers()?;
         assert!(self.dirty);
 
-        // Update CRC on the unencoded data.
-        self.crc.update(buf);
+        match self.profile {
+            None | Some(Profile::RFC4880) => {
+                // Update CRC on the unencoded data.
+                self.crc.update(buf);
+            },
+
+            Some(Profile::RFC9580) => (),
+        }
 
         let mut input = buf;
         let mut written = 0;
@@ -508,6 +569,7 @@ impl<W: Write> Write for Writer<W> {
 
 /// How an ArmorReader should act.
 #[derive(Debug, Clone, Copy, PartialEq)]
+#[non_exhaustive]
 pub enum ReaderMode {
     /// Makes the armor reader tolerant of simple errors.
     ///
@@ -614,21 +676,6 @@ impl Default for CSFTransformer {
 }
 
 impl<'a> Reader<'a> {
-    /// Constructs a new filter for the given type of data.
-    ///
-    /// This function is deprecated and will be removed in version
-    /// 2.0. Please use [`Reader::from_reader`][].
-    #[deprecated = "Use Reader::from_reader. `new` will be removed in version 2.0"]
-    pub fn new<R, M>(inner: R, mode: M) -> Self
-        where R: 'a + Read + Send + Sync,
-              M: Into<Option<ReaderMode>>
-    {
-        Self::from_cookie_reader(
-            Box::new(buffered_reader::Generic::with_cookie(inner, None,
-                                                           Default::default())),
-            mode, Default::default())
-    }
-
     /// Constructs a new `Reader` from the given `BufferedReader`.
     pub fn from_buffered_reader<R, M>(reader: R, mode: M) -> Result<Self>
     where
@@ -658,7 +705,7 @@ impl<'a> Reader<'a> {
     /// This mode of operation is CPU intense, particularly on large
     /// text files.
     ///
-    ///   [ASCII Armor]: https://tools.ietf.org/html/rfc4880#section-6.2
+    ///   [ASCII Armor]: https://www.rfc-editor.org/rfc/rfc9580.html#section-6.2
     ///
     /// # Examples
     ///
@@ -845,8 +892,13 @@ impl<'a> Reader<'a> {
         // The range of the first 6 bits of a message is limited.
         // Save cpu cycles by only considering base64 data that starts
         // with one of those characters.
-        lazy_static::lazy_static!{
-            static ref START_CHARS_VERY_TOLERANT: Vec<u8> = {
+        fn start_chars_very_tolerant() -> &'static [u8] {
+            use std::sync::OnceLock;
+
+            static START_CHARS_VERY_TOLERANT: OnceLock<Vec<u8>>
+                = OnceLock::new();
+
+            START_CHARS_VERY_TOLERANT.get_or_init(|| {
                 let mut valid_start = Vec::new();
                 for &tag in &[ Tag::PKESK, Tag::SKESK,
                               Tag::OnePassSig, Tag::Signature,
@@ -883,9 +935,16 @@ impl<'a> Reader<'a> {
                 valid_start.sort_unstable();
                 valid_start.dedup();
                 valid_start
-            };
+            })
+        }
 
-            static ref START_CHARS_TOLERANT: Vec<u8> = {
+        fn start_chars_tolerant() -> &'static [u8] {
+            use std::sync::OnceLock;
+
+            static START_CHARS_TOLERANT: OnceLock<Vec<u8>>
+                = OnceLock::new();
+
+            START_CHARS_TOLERANT.get_or_init(|| {
                 let mut valid_start = Vec::new();
                 // Add all first bytes of Unicode characters from the
                 // "Dash Punctuation" category.
@@ -901,16 +960,16 @@ impl<'a> Reader<'a> {
                 valid_start.sort_unstable();
                 valid_start.dedup();
                 valid_start
-            };
+            })
         }
 
         // Look for the Armor Header Line, skipping any garbage in the
         // process.
         let mut found_blob = false;
         let start_chars = if self.mode != ReaderMode::VeryTolerant {
-            &START_CHARS_TOLERANT[..]
+            start_chars_tolerant()
         } else {
-            &START_CHARS_VERY_TOLERANT[..]
+            start_chars_very_tolerant()
         };
 
         let mut lines = 0;
@@ -1461,7 +1520,8 @@ impl<'a> Reader<'a> {
                 }
 
                 // Now, we have the whole text.
-                let mut literal = Literal::new(DataFormat::Text);
+                // XXX: We optimistically assume that it is UTF-8 encoded.
+                let mut literal = Literal::new(DataFormat::Unicode);
                 literal.set_body(text);
                 literal
             };
@@ -1505,7 +1565,7 @@ impl<'a> Reader<'a> {
             let mut opss = Vec::with_capacity(sigs.len());
             for p in sigs.iter().rev() {
                 if let Packet::Signature(sig) = p {
-                    if let Ok(ops) = OnePassSig3::try_from(sig) {
+                    if let Ok(ops) = OnePassSig::try_from(sig) {
                         opss.push(ops);
                     }
                 }
@@ -2355,10 +2415,12 @@ mod test {
             reader.read_to_end(&mut buf)?;
 
             let message = crate::Message::from_bytes(&buf)?;
-            assert_eq!(message.children().count(), 3);
+            assert_eq!(message.packets().children().count(), 3);
 
             // First, an one-pass-signature packet.
-            if let Some(Packet::OnePassSig(ops)) = message.path_ref(&[0]) {
+            if let Some(Packet::OnePassSig(ops)) =
+                message.packets().path_ref(&[0])
+            {
                 assert_eq!(ops.hash_algo(), hash);
             } else {
                 panic!("expected an OPS packet");
@@ -2368,7 +2430,9 @@ mod test {
             assert_eq!(message.body().unwrap().body(), reference.as_ref());
 
             // And, the signature.
-            if let Some(Packet::Signature(sig)) = message.path_ref(&[2]) {
+            if let Some(Packet::Signature(sig)) =
+                message.packets().path_ref(&[2])
+            {
                 assert_eq!(sig.hash_algo(), hash);
             } else {
                 panic!("expected an signature packet");
@@ -2405,19 +2469,9 @@ mod test {
               assert_eq!(reference.pop(), Some(b'\n'));
               reference
           }, HashAlgorithm::SHA256)?;
-        f(crate::tests::message("a-cypherpunks-manifesto.txt.cleartext.sig"),
-          {
-              // The test vector, created by GnuPG, does not preserve
-              // the final newline.
-              //
-              // The transformation process trims trailing whitespace,
-              // and the manifesto has a trailing whitespace right at
-              // the end.
-              let mut manifesto = crate::tests::manifesto().to_vec();
-              assert_eq!(manifesto.pop(), Some(b'\n'));
-              assert_eq!(manifesto.pop(), Some(b' '));
-              manifesto
-          }, HashAlgorithm::SHA256)?;
+        f(crate::tests::file("crypto-refresh/cleartext-signed-message.txt"),
+          crate::tests::file("crypto-refresh/cleartext-signed-message.txt.plain"),
+          HashAlgorithm::SHA512)?;
         Ok(())
     }
 }
