@@ -5,10 +5,12 @@ use crate::packet;
 use crate::packet::{
     Key,
     key::Key4,
-    key::KeyRole,
+    key::Key6,
+    key::UnspecifiedRole,
     key::SecretKey as KeySecretKey,
     key::SecretParts as KeySecretParts,
 };
+use crate::Profile;
 use crate::Result;
 use crate::packet::Signature;
 use crate::packet::signature::{
@@ -47,20 +49,21 @@ pub use key::{
 ///
 /// # fn main() -> openpgp::Result<()> {
 /// let (ecc, _) =
-///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+///     CertBuilder::general_purpose(Some("alice@example.org"))
 ///         .set_cipher_suite(CipherSuite::Cv25519)
 ///         .generate()?;
-/// assert_eq!(ecc.primary_key().pk_algo(), PublicKeyAlgorithm::EdDSA);
+/// assert_eq!(ecc.primary_key().key().pk_algo(), PublicKeyAlgorithm::EdDSA);
 ///
 /// let (rsa, _) =
-///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+///     CertBuilder::general_purpose(Some("alice@example.org"))
 ///         .set_cipher_suite(CipherSuite::RSA4k)
 ///         .generate()?;
-/// assert_eq!(rsa.primary_key().pk_algo(), PublicKeyAlgorithm::RSAEncryptSign);
+/// assert_eq!(rsa.primary_key().key().pk_algo(), PublicKeyAlgorithm::RSAEncryptSign);
 /// # Ok(())
 /// # }
 /// ```
-#[derive(Clone, Copy, PartialEq, PartialOrd, Eq, Ord, Debug)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+#[non_exhaustive]
 pub enum CipherSuite {
     /// EdDSA and ECDH over Curve25519 with SHA512 and AES256
     Cv25519,
@@ -151,10 +154,19 @@ impl CipherSuite {
         Ok(())
     }
 
-    fn generate_key<K, R>(self, flags: K)
-        -> Result<Key<KeySecretParts, R>>
-        where R: KeyRole,
-              K: AsRef<KeyFlags>,
+    fn generate_key<K>(self, flags: K, profile: Profile)
+        -> Result<Key<KeySecretParts, UnspecifiedRole>>
+        where K: AsRef<KeyFlags>,
+    {
+        match profile {
+            Profile::RFC9580 => Ok(self.generate_v6_key(flags)?.into()),
+            Profile::RFC4880 => Ok(self.generate_v4_key(flags)?.into()),
+        }
+    }
+
+    fn generate_v4_key<K>(self, flags: K)
+        -> Result<Key4<KeySecretParts, UnspecifiedRole>>
+        where K: AsRef<KeyFlags>,
     {
         use crate::types::Curve;
 
@@ -199,7 +211,69 @@ impl CipherSuite {
                             .into()),
                 }
             },
-        }.map(|key| key.into())
+        }
+    }
+
+    fn generate_v6_key<K>(self, flags: K)
+        -> Result<Key6<KeySecretParts, UnspecifiedRole>>
+        where K: AsRef<KeyFlags>,
+    {
+        use crate::types::Curve;
+
+        let flags = flags.as_ref();
+        let sign = flags.for_certification() || flags.for_signing()
+            || flags.for_authentication();
+        let encrypt = flags.for_transport_encryption()
+            || flags.for_storage_encryption();
+
+        match self {
+            CipherSuite::Cv25519 => match (sign, encrypt) {
+                (true, false) => Key6::generate_ed25519(),
+                (false, true) => Key6::generate_x25519(),
+                (true, true) =>
+                    Err(Error::InvalidOperation(
+                        "Can't use key for encryption and signing".into())
+                        .into()),
+                (false, false) =>
+                    Err(Error::InvalidOperation(
+                        "No key flags set".into())
+                        .into()),
+            },
+            CipherSuite::RSA2k =>
+                Key6::generate_rsa(2048),
+            CipherSuite::RSA3k =>
+                Key6::generate_rsa(3072),
+            CipherSuite::RSA4k =>
+                Key6::generate_rsa(4096),
+            CipherSuite::P256 | CipherSuite::P384 | CipherSuite::P521 => {
+                let curve = match self {
+                    CipherSuite::Cv25519 if sign => Curve::Ed25519,
+                    CipherSuite::Cv25519 if encrypt => Curve::Cv25519,
+                    CipherSuite::Cv25519 => {
+                        return Err(Error::InvalidOperation(
+                            "No key flags set".into())
+                            .into());
+                    }
+                    CipherSuite::P256 => Curve::NistP256,
+                    CipherSuite::P384 => Curve::NistP384,
+                    CipherSuite::P521 => Curve::NistP521,
+                    _ => unreachable!(),
+                };
+
+                match (sign, encrypt) {
+                    (true, false) => Key6::generate_ecc(true, curve),
+                    (false, true) => Key6::generate_ecc(false, curve),
+                    (true, true) =>
+                        Err(Error::InvalidOperation(
+                            "Can't use key for encryption and signing".into())
+                            .into()),
+                    (false, false) =>
+                        Err(Error::InvalidOperation(
+                            "No key flags set".into())
+                            .into()),
+                }
+            },
+        }
     }
 }
 
@@ -272,7 +346,7 @@ assert_send_and_sync!(KeyBlueprint);
 ///
 /// # fn main() -> openpgp::Result<()> {
 /// let (cert, rev) =
-///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+///     CertBuilder::general_purpose(Some("alice@example.org"))
 ///         .generate()?;
 /// # Ok(())
 /// # }
@@ -280,6 +354,10 @@ assert_send_and_sync!(KeyBlueprint);
 pub struct CertBuilder<'a> {
     creation_time: Option<std::time::SystemTime>,
     ciphersuite: CipherSuite,
+    profile: Profile,
+
+    /// Advertised features.
+    features: Features,
     primary: KeyBlueprint,
     subkeys: Vec<(Option<SignatureBuilder>, KeyBlueprint)>,
     userids: Vec<(Option<SignatureBuilder>, packet::UserID)>,
@@ -332,6 +410,8 @@ impl CertBuilder<'_> {
         CertBuilder {
             creation_time: None,
             ciphersuite: CipherSuite::default(),
+            profile: Default::default(),
+            features: Features::sequoia(),
             primary: KeyBlueprint{
                 flags: KeyFlags::empty().set_certification(),
                 validity: None,
@@ -366,20 +446,25 @@ impl CertBuilder<'_> {
     ///
     /// # fn main() -> openpgp::Result<()> {
     /// let (cert, rev) =
-    ///     CertBuilder::general_purpose(None,
-    ///                                  Some("Alice Lovelace <alice@example.org>"))
+    ///     CertBuilder::general_purpose(["Alice Lovelace", "<alice@example.org>"])
+    ///         .generate()?;
+    /// # assert_eq!(cert.keys().count(), 3);
+    /// # assert_eq!(cert.userids().count(), 2);
+    ///
+    /// let (cert, rev) =
+    ///     CertBuilder::general_purpose(["Alice Lovelace <alice@example.org>"])
     ///         .generate()?;
     /// # assert_eq!(cert.keys().count(), 3);
     /// # assert_eq!(cert.userids().count(), 1);
     /// # Ok(())
     /// # }
     /// ```
-    pub fn general_purpose<C, U>(ciphersuite: C, userid: Option<U>) -> Self
-        where C: Into<Option<CipherSuite>>,
-              U: Into<packet::UserID>
+    pub fn general_purpose<U>(userids: U) -> Self
+    where
+        U: IntoIterator,
+        U::Item: Into<packet::UserID>,
     {
         let mut builder = Self::new()
-            .set_cipher_suite(ciphersuite.into().unwrap_or_default())
             .set_primary_key_flags(KeyFlags::empty().set_certification())
             .set_validity_period(
                 time::Duration::new(3 * 52 * 7 * 24 * 60 * 60, 0))
@@ -387,8 +472,9 @@ impl CertBuilder<'_> {
             .add_subkey(KeyFlags::empty()
                         .set_transport_encryption()
                         .set_storage_encryption(), None, None);
-        if let Some(u) = userid.map(Into::into) {
-            builder = builder.add_userid(u);
+
+        for u in userids {
+            builder = builder.add_userid(u.into());
         }
 
         builder
@@ -442,8 +528,7 @@ impl CertBuilder<'_> {
     /// let t = SystemTime::from(Timestamp::try_from(t)?);
     ///
     /// let (cert, rev) =
-    ///     CertBuilder::general_purpose(None,
-    ///                                  Some("Alice Lovelace <alice@example.org>"))
+    ///     CertBuilder::general_purpose(Some("Alice Lovelace <alice@example.org>"))
     ///         .set_creation_time(t)
     ///         .generate()?;
     /// assert_eq!(cert.primary_key().self_signatures().nth(0).unwrap()
@@ -501,16 +586,16 @@ impl CertBuilder<'_> {
     ///
     /// # fn main() -> openpgp::Result<()> {
     /// let (ecc, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .set_cipher_suite(CipherSuite::Cv25519)
     ///         .generate()?;
-    /// assert_eq!(ecc.primary_key().pk_algo(), PublicKeyAlgorithm::EdDSA);
+    /// assert_eq!(ecc.primary_key().key().pk_algo(), PublicKeyAlgorithm::EdDSA);
     ///
     /// let (rsa, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .set_cipher_suite(CipherSuite::RSA2k)
     ///         .generate()?;
-    /// assert_eq!(rsa.primary_key().pk_algo(), PublicKeyAlgorithm::RSAEncryptSign);
+    /// assert_eq!(rsa.primary_key().key().pk_algo(), PublicKeyAlgorithm::RSAEncryptSign);
     /// # Ok(())
     /// # }
     /// ```
@@ -523,11 +608,11 @@ impl CertBuilder<'_> {
     ///
     /// This method controls whether the certificate is exportable.
     /// If the certificate builder is configured to make a
-    /// non-exportable certificate, then all of the signatures that it
-    /// creates include the an [Exportable Certification] subpacket
+    /// non-exportable certificate, then all the signatures that it
+    /// creates include the [Exportable Certification] subpacket
     /// that is set to `false`.
     ///
-    /// [Exportable Certification]: https://datatracker.ietf.org/doc/html/rfc4880#section-5.2.3.11
+    /// [Exportable Certification]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.19
     ///
     /// # Examples
     ///
@@ -544,7 +629,7 @@ impl CertBuilder<'_> {
     ///
     /// # fn main() -> openpgp::Result<()> {
     /// let (cert, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .set_exportable(false)
     ///         .generate()?;
     /// let mut exported = Vec::new();
@@ -562,6 +647,72 @@ impl CertBuilder<'_> {
         self
     }
 
+    /// Sets the version of OpenPGP to generate keys for.
+    ///
+    /// Supported are [`Profile::RFC9580`] which will generate version
+    /// 6 keys, and [`Profile::RFC4880`] which will version 4 keys.
+    /// By default, we generate version 6 keys.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::cert::prelude::*;
+    ///
+    /// # fn main() -> openpgp::Result<()> {
+    /// let (key, _) =
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
+    ///         .set_profile(openpgp::Profile::RFC9580)?
+    ///         .generate()?;
+    /// assert_eq!(key.primary_key().key().version(), 6);
+    ///
+    /// let (key, _) =
+    ///     CertBuilder::general_purpose(Some("bob@example.org"))
+    ///         .set_profile(openpgp::Profile::RFC4880)?
+    ///         .generate()?;
+    /// assert_eq!(key.primary_key().key().version(), 4);
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn set_profile(mut self, profile: Profile) -> Result<Self> {
+        self.profile = profile;
+        Ok(self)
+    }
+
+    /// Sets the features the certificate will advertise.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::types::Features;
+    ///
+    /// # fn main() -> openpgp::Result<()> {
+    /// let (key, _) =
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
+    ///         .set_profile(openpgp::Profile::RFC9580)?
+    ///         .generate()?;
+    /// assert_eq!(key.primary_key().key().version(), 6);
+    ///
+    /// // Bob is old-school: he needs this key to work on an old
+    /// // phone.  Therefore, he doesn't want to get SEIPDv2 messages,
+    /// // which his phone doesn't handle.  He advertises support for
+    /// // SEIPDv1 only.
+    /// let (key, _) =
+    ///     CertBuilder::general_purpose(Some("bob@example.org"))
+    ///         .set_profile(openpgp::Profile::RFC4880)?
+    ///         .set_features(Features::empty().set_seipdv1())?
+    ///         .generate()?;
+    /// assert_eq!(key.primary_key().key().version(), 4);
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn set_features(mut self, features: Features) -> Result<Self> {
+        self.features = features;
+        Ok(self)
+    }
+
     /// Adds a User ID.
     ///
     /// Adds a User ID to the certificate.  The first User ID that is
@@ -570,7 +721,7 @@ impl CertBuilder<'_> {
     /// ID flag] set.
     ///
     /// [`CertBuilder::general_purpose`]: CertBuilder::general_purpose()
-    /// [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    /// [primary User ID flag]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.27
     ///
     /// # Examples
     ///
@@ -584,15 +735,14 @@ impl CertBuilder<'_> {
     /// let p = &StandardPolicy::new();
     ///
     /// let (cert, rev) =
-    ///     CertBuilder::general_purpose(None,
-    ///                                  Some("Alice Lovelace <alice@example.org>"))
+    ///     CertBuilder::general_purpose(Some("Alice Lovelace <alice@example.org>"))
     ///         .add_userid("Alice Lovelace <alice@lovelace.name>")
     ///         .generate()?;
     ///
     /// assert_eq!(cert.userids().count(), 2);
     /// let mut userids = cert.with_policy(p, None)?.userids().collect::<Vec<_>>();
     /// // Sort lexicographically.
-    /// userids.sort_by(|a, b| a.value().cmp(b.value()));
+    /// userids.sort_by(|a, b| a.userid().value().cmp(b.userid().value()));
     /// assert_eq!(userids[0].userid(),
     ///            &UserID::from("Alice Lovelace <alice@example.org>"));
     /// assert_eq!(userids[1].userid(),
@@ -646,7 +796,7 @@ impl CertBuilder<'_> {
     ///   [`PersonaCertification`]: crate::types::SignatureType::PersonaCertification
     ///   [`CasualCertification`]: crate::types::SignatureType::CasualCertification
     ///   [`PositiveCertification`]: crate::types::SignatureType::PositiveCertification
-    ///   [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    ///   [primary User ID flag]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.27
     ///
     /// # Examples
     ///
@@ -663,7 +813,7 @@ impl CertBuilder<'_> {
     /// #
     /// let (cert, revocation_cert) =
     ///     CertBuilder::general_purpose(
-    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///         Some("Alice Lovelace <alice@example.org>"))
     ///     .add_userid_with(
     ///         "trinity",
     ///         SignatureBuilder::new(SignatureType::CasualCertification)
@@ -675,7 +825,7 @@ impl CertBuilder<'_> {
     /// assert_eq!(cert.userids().count(), 2);
     /// let mut userids = cert.with_policy(policy, None)?.userids().collect::<Vec<_>>();
     /// // Sort lexicographically.
-    /// userids.sort_by(|a, b| a.value().cmp(b.value()));
+    /// userids.sort_by(|a, b| a.userid().value().cmp(b.userid().value()));
     /// assert_eq!(userids[0].userid(),
     ///            &UserID::from("Alice Lovelace <alice@example.org>"));
     /// assert_eq!(userids[1].userid(),
@@ -715,7 +865,7 @@ impl CertBuilder<'_> {
     /// this interface or another interface, will have the [primary
     /// User ID flag] set.
     ///
-    /// [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    /// [primary User ID flag]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.27
     ///
     /// # Examples
     ///
@@ -823,7 +973,7 @@ impl CertBuilder<'_> {
     ///   [`PersonaCertification`]: crate::types::SignatureType::PersonaCertification
     ///   [`CasualCertification`]: crate::types::SignatureType::CasualCertification
     ///   [`PositiveCertification`]: crate::types::SignatureType::PositiveCertification
-    ///   [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
+    ///   [primary User ID flag]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.27
     ///
     /// # Examples
     ///
@@ -847,7 +997,7 @@ impl CertBuilder<'_> {
     /// #   UserAttribute::new(&[Subpacket::Unknown(7, vec![7; 7].into())])?;
     /// let (cert, revocation_cert) =
     ///     CertBuilder::general_purpose(
-    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///         Some("Alice Lovelace <alice@example.org>"))
     ///     .add_user_attribute_with(
     ///         user_attribute,
     ///         SignatureBuilder::new(SignatureType::CasualCertification)
@@ -999,7 +1149,7 @@ impl CertBuilder<'_> {
                         None, None)
     }
 
-    /// Adds an certification-capable subkey.
+    /// Adds a certification-capable subkey.
     ///
     /// The key uses the default cipher suite (see
     /// [`CertBuilder::set_cipher_suite`]), and is not set to expire.
@@ -1178,7 +1328,7 @@ impl CertBuilder<'_> {
     /// # use openpgp::types::*;
     /// let (cert, revocation_cert) =
     ///     CertBuilder::general_purpose(
-    ///         None, Some("Alice Lovelace <alice@example.org>"))
+    ///         Some("Alice Lovelace <alice@example.org>"))
     ///     .add_subkey_with(
     ///         KeyFlags::empty().set_signing(), None, None,
     ///         SignatureBuilder::new(SignatureType::SubkeyBinding)
@@ -1242,8 +1392,7 @@ impl CertBuilder<'_> {
     /// let p = &StandardPolicy::new();
     ///
     /// let (cert, rev) =
-    ///     CertBuilder::general_purpose(None,
-    ///                                  Some("Alice Lovelace <alice@example.org>"))
+    ///     CertBuilder::general_purpose(Some("Alice Lovelace <alice@example.org>"))
     ///         .set_primary_key_flags(KeyFlags::empty().set_signing())
     ///         .generate()?;
     ///
@@ -1272,13 +1421,12 @@ impl CertBuilder<'_> {
     /// # fn main() -> Result<()> {
     /// // Make the certificate expire in 10 minutes.
     /// let (cert, rev) =
-    ///     CertBuilder::general_purpose(None,
-    ///                                  Some("Alice Lovelace <alice@example.org>"))
+    ///     CertBuilder::general_purpose(Some("Alice Lovelace <alice@example.org>"))
     ///         .set_password(Some("1234".into()))
     ///         .generate()?;
     ///
     /// for ka in cert.keys() {
-    ///     assert!(ka.has_secret());
+    ///     assert!(ka.key().has_secret());
     /// }
     /// # Ok(()) }
     /// ```
@@ -1355,10 +1503,10 @@ impl CertBuilder<'_> {
     /// let p = &StandardPolicy::new();
     ///
     /// let (alice, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .generate()?;
     /// let (bob, _) =
-    ///     CertBuilder::general_purpose(None, Some("bob@example.org"))
+    ///     CertBuilder::general_purpose(Some("bob@example.org"))
     ///         .set_revocation_keys(vec![(&alice).into()])
     ///         .generate()?;
     ///
@@ -1389,11 +1537,11 @@ impl CertBuilder<'_> {
     /// let p = &StandardPolicy::new();
     ///
     /// let (alice, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .generate()?;
     /// # Ok(()) }
     /// ```
-    pub fn generate(self) -> Result<(Cert, Signature)> {
+    pub fn generate(mut self) -> Result<(Cert, Signature)> {
         use crate::Packet;
         use crate::types::ReasonForRevocation;
 
@@ -1413,7 +1561,9 @@ impl CertBuilder<'_> {
             Packet::SecretKey({
                 let mut primary = primary.clone();
                 if let Some(ref password) = self.password {
-                    primary.secret_mut().encrypt_in_place(password)?;
+                    let (k, mut secret) = primary.take_secret();
+                    secret.encrypt_in_place(&k, password)?;
+                    primary = k.add_secret(secret).0;
                 }
                 primary
             }),
@@ -1439,12 +1589,12 @@ impl CertBuilder<'_> {
         let mut emitted_primary_user_thing = false;
 
         // Sign UserIDs.
-        for (template, uid) in self.userids.into_iter() {
+        for (template, uid) in std::mem::take(&mut self.userids).into_iter() {
             let sig = template.unwrap_or_else(
                 || SignatureBuilder::new(SignatureType::PositiveCertification));
             let sig = Self::signature_common(sig, creation_time,
                                              self.exportable)?;
-            let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
+            let mut sig = self.add_primary_key_metadata(sig)?;
 
             // Make sure we mark exactly one User ID or Attribute as
             // primary.
@@ -1470,12 +1620,12 @@ impl CertBuilder<'_> {
         }
 
         // Sign UserAttributes.
-        for (template, ua) in self.user_attributes.into_iter() {
+        for (template, ua) in std::mem::take(&mut self.user_attributes) {
             let sig = template.unwrap_or_else(
                 || SignatureBuilder::new(SignatureType::PositiveCertification));
             let sig = Self::signature_common(
                 sig, creation_time, self.exportable)?;
-            let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
+            let mut sig = self.add_primary_key_metadata(sig)?;
 
             // Make sure we mark exactly one User ID or Attribute as
             // primary.
@@ -1505,7 +1655,8 @@ impl CertBuilder<'_> {
             let flags = &blueprint.flags;
             let mut subkey = blueprint.ciphersuite
                 .unwrap_or(self.ciphersuite)
-                .generate_key(flags)?;
+                .generate_key(flags, self.profile)?
+                .role_into_subordinate();
             subkey.set_creation_time(creation_time)?;
 
             let sig = template.unwrap_or_else(
@@ -1534,14 +1685,16 @@ impl CertBuilder<'_> {
             let signature = subkey.bind(&mut signer, &cert, builder)?;
 
             if let Some(ref password) = self.password {
-                subkey.secret_mut().encrypt_in_place(password)?;
+                let (k, mut secret) = subkey.take_secret();
+                secret.encrypt_in_place(&k, password)?;
+                subkey = k.add_secret(secret).0;
             }
             acc.push(subkey.into());
             acc.push(signature.into());
         }
 
         // Now add the new components and canonicalize once.
-        let cert = cert.insert_packets(acc)?;
+        let cert = cert.insert_packets(acc)?.0;
 
         let revocation = CertRevocationBuilder::new()
             .set_signature_creation_time(creation_time)?
@@ -1562,15 +1715,19 @@ impl CertBuilder<'_> {
     {
         let mut key = self.primary.ciphersuite
             .unwrap_or(self.ciphersuite)
-            .generate_key(KeyFlags::empty().set_certification())?;
+            .generate_key(KeyFlags::empty().set_certification(),
+                          self.profile)?
+            .role_into_primary();
         key.set_creation_time(creation_time)?;
         let sig = SignatureBuilder::new(SignatureType::DirectKey);
         let sig = Self::signature_common(
             sig, creation_time, self.exportable)?;
-        let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
+        let mut sig = self.add_primary_key_metadata(sig)?;
 
         if let Some(ref revocation_keys) = self.revocation_keys {
-            sig = sig.set_revocation_key(revocation_keys.clone())?;
+            for k in revocation_keys.into_iter().cloned() {
+                sig = sig.add_revocation_key(k)?;
+            }
         }
 
         let mut signer = key.clone().into_keypair()
@@ -1598,14 +1755,14 @@ impl CertBuilder<'_> {
 
 
     /// Adds primary key metadata to the signature.
-    fn add_primary_key_metadata(builder: SignatureBuilder,
-                                primary: &KeyBlueprint)
+    fn add_primary_key_metadata(&self,
+                                builder: SignatureBuilder)
                                 -> Result<SignatureBuilder>
     {
         builder
-            .set_features(Features::sequoia())?
-            .set_key_flags(primary.flags.clone())?
-            .set_key_validity_period(primary.validity)?
+            .set_features(self.features.clone())?
+            .set_key_flags(self.primary.flags.clone())?
+            .set_key_validity_period(self.primary.validity)?
             .set_preferred_hash_algorithms(vec![
                 HashAlgorithm::SHA512,
                 HashAlgorithm::SHA256,
@@ -1680,14 +1837,14 @@ mod tests {
             .set_cipher_suite(CipherSuite::RSA3k)
             .set_cipher_suite(CipherSuite::Cv25519)
             .generate().unwrap();
-        assert_eq!(cert1.primary_key().pk_algo(), PublicKeyAlgorithm::EdDSA);
+        assert_eq!(cert1.primary_key().key().pk_algo(), PublicKeyAlgorithm::EdDSA);
 
         let (cert2, _) = CertBuilder::new()
             .set_cipher_suite(CipherSuite::RSA3k)
             .add_userid("test2@example.com")
             .add_transport_encryption_subkey()
             .generate().unwrap();
-        assert_eq!(cert2.primary_key().pk_algo(),
+        assert_eq!(cert2.primary_key().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
         assert_eq!(cert2.subkeys().next().unwrap().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
@@ -1699,7 +1856,7 @@ mod tests {
         let (cert1, _) = CertBuilder::new()
             .add_userid("test2@example.com")
             .generate().unwrap();
-        assert_eq!(cert1.primary_key().pk_algo(),
+        assert_eq!(cert1.primary_key().key().pk_algo(),
                    PublicKeyAlgorithm::EdDSA);
         assert!(cert1.subkeys().next().is_none());
         assert!(cert1.with_policy(p, None).unwrap().primary_userid().unwrap()
@@ -1726,7 +1883,7 @@ mod tests {
             .add_subkey(KeyFlags::empty().set_certification(), None, None)
             .generate().unwrap();
         let sig_pkts = cert1.subkeys().next().unwrap().bundle()
-            .self_signatures2().next().unwrap().hashed_area();
+            .self_signatures().next().unwrap().hashed_area();
 
         match sig_pkts.subpacket(SubpacketTag::KeyFlags).unwrap().value() {
             SubpacketValue::KeyFlags(ref ks) => assert!(ks.for_certification()),
@@ -1746,7 +1903,7 @@ mod tests {
         assert_eq!(cert.revocation_status(p, None),
                    RevocationStatus::NotAsFarAsWeKnow);
 
-        let cert = cert.insert_packets(revocation.clone()).unwrap();
+        let cert = cert.insert_packets(revocation.clone()).unwrap().0;
         assert_eq!(cert.revocation_status(p, None),
                    RevocationStatus::Revoked(vec![ &revocation ]));
     }
@@ -1771,14 +1928,12 @@ mod tests {
             .set_cipher_suite(CipherSuite::Cv25519)
             .set_password(Some(String::from("streng geheim").into()))
             .generate().unwrap();
-        assert!(cert.primary_key().optional_secret().unwrap().is_encrypted());
+        assert!(cert.primary_key().key().optional_secret().unwrap().is_encrypted());
     }
 
     #[test]
     fn all_ciphersuites() {
-        use self::CipherSuite::*;
-
-        for cs in vec![Cv25519, RSA3k, P256, P384, P521, RSA2k, RSA4k]
+        for cs in CipherSuite::variants()
             .into_iter().filter(|cs| cs.is_supported().is_ok())
         {
             assert!(CertBuilder::new()
@@ -1804,7 +1959,7 @@ mod tests {
             .generate().unwrap();
 
         let key = cert.primary_key().key();
-        let sig = &cert.primary_key().bundle().self_signatures2().next().unwrap();
+        let sig = &cert.primary_key().bundle().self_signatures().next().unwrap();
         assert!(sig.key_alive(key, now).is_ok());
         assert!(sig.key_alive(key, now + 590 * s).is_ok());
         assert!(! sig.key_alive(key, now + 610 * s).is_ok());
@@ -1834,7 +1989,7 @@ mod tests {
             .add_signing_subkey()
             .generate().unwrap();
 
-        assert_eq!(cert.primary_key().creation_time(), UNIX_EPOCH);
+        assert_eq!(cert.primary_key().key().creation_time(), UNIX_EPOCH);
         assert_eq!(cert.primary_key().with_policy(p, None).unwrap()
                    .binding_signature()
                    .signature_creation_time().unwrap(), UNIX_EPOCH);
@@ -1877,12 +2032,12 @@ mod tests {
         ];
 
         let (cert,_)
-            = CertBuilder::general_purpose(None, Some("alice@example.org"))
+            = CertBuilder::general_purpose(Some("alice@example.org"))
             .set_revocation_keys(revokers.clone())
             .generate()?;
         let cert = cert.with_policy(p, None)?;
 
-        assert_eq!(cert.revocation_keys(None).collect::<HashSet<_>>(),
+        assert_eq!(cert.revocation_keys().collect::<HashSet<_>>(),
                    revokers.iter().collect::<HashSet<_>>());
 
         // Do it again, with a key that has no User IDs.
@@ -1892,7 +2047,7 @@ mod tests {
         let cert = cert.with_policy(p, None)?;
         assert!(cert.primary_userid().is_err());
 
-        assert_eq!(cert.revocation_keys(None).collect::<HashSet<_>>(),
+        assert_eq!(cert.revocation_keys().collect::<HashSet<_>>(),
                    revokers.iter().collect::<HashSet<_>>());
 
         // The designated revokers on all signatures should be
@@ -1906,15 +2061,16 @@ mod tests {
 
         // Add a newer direct key signature.
         use crate::crypto::hash::Hash;
-        let mut hash = HashAlgorithm::SHA512.context()?;
-        cert.primary_key().hash(&mut hash);
         let mut primary_signer =
             cert.primary_key().key().clone().parts_into_secret()?
             .into_keypair()?;
+        let mut hash = HashAlgorithm::SHA512.context()?
+            .for_signature(primary_signer.public().version());
+        cert.primary_key().key().hash(&mut hash)?;
         let sig = signature::SignatureBuilder::new(SignatureType::DirectKey)
             .set_signature_creation_time(then)?
             .sign_hash(&mut primary_signer, hash)?;
-        let cert = cert.insert_packets(sig)?;
+        let cert = cert.insert_packets(sig)?.0;
 
         assert!(cert.with_policy(p, then)?.primary_userid().is_err());
         assert_eq!(cert.revocation_keys(p).collect::<HashSet<_>>(),
@@ -1927,7 +2083,7 @@ mod tests {
     #[test]
     fn primary_user_things() -> Result<()> {
         fn count_primary_user_things(c: Cert) -> usize {
-            c.into_packets2().map(|p| match p {
+            c.into_packets().map(|p| match p {
                 Packet::Signature(s) if s.primary_userid().unwrap_or(false)
                     => 1,
                 _ => 0,
@@ -2003,7 +2159,7 @@ mod tests {
                 SignatureBuilder::new(positive).set_primary_userid(true)?)?
             .generate()?;
         let vc = c.with_policy(p, None)?;
-        assert_eq!(vc.primary_userid()?.value(), b"bar");
+        assert_eq!(vc.primary_userid()?.userid().value(), b"bar");
         assert_eq!(count_primary_user_things(c), 1);
 
         Ok(())
@@ -2015,12 +2171,12 @@ mod tests {
         // nothing is exported.
 
         let (cert, _) =
-            CertBuilder::general_purpose(None, Some("alice@example.org"))
+            CertBuilder::general_purpose(Some("alice@example.org"))
             .set_exportable(false)
             .generate()?;
 
         let (bob, _) =
-            CertBuilder::general_purpose(None, Some("bob@example.org"))
+            CertBuilder::general_purpose(Some("bob@example.org"))
             .generate()?;
 
         // Have Bob certify Alice's primary User ID with an exportable
@@ -2029,10 +2185,11 @@ mod tests {
         let mut keypair = bob.primary_key().key().clone()
             .parts_into_secret()?.into_keypair()?;
         let certification = cert.userids().nth(0).unwrap()
+            .userid()
             .certify(&mut keypair, &cert,
                      SignatureType::PositiveCertification,
                      None, None)?;
-        let cert = cert.insert_packets(certification)?;
+        let cert = cert.insert_packets(certification)?.0;
 
         macro_rules! check {
             ($cert: expr, $export: ident, $expected: expr) => {
@@ -2069,15 +2226,16 @@ mod tests {
         check!(cert.as_tsk().armored(), export, 0);
         check!(cert.as_tsk().armored(), serialize, 1);
 
-        // Have Alice add a exportable self signature.  Now her's
+        // Have Alice add an exportable self signature.  Now her
         // certificate should be exportable.
         let mut keypair = cert.primary_key().key().clone()
             .parts_into_secret()?.into_keypair()?;
         let certification = cert.userids().nth(0).unwrap()
+            .userid()
             .certify(&mut keypair, &cert,
                      SignatureType::PositiveCertification,
                      None, None)?;
-        let cert = cert.insert_packets(certification)?;
+        let cert = cert.insert_packets(certification)?.0;
 
         macro_rules! check {
             ($cert: expr, $export: ident, $expected: expr) => {
