@@ -38,9 +38,11 @@ impl Asymmetric for super::Backend {
         use PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match algo {
+            X25519 | Ed25519 |
             RSAEncryptSign | RSAEncrypt | RSASign | DSA | ECDH | ECDSA | EdDSA |
             ElGamalEncrypt | ElGamalEncryptSign
                 => true,
+            X448 | Ed448 |
             Private(_) | Unknown(_)
                 => false,
         }
@@ -50,9 +52,7 @@ impl Asymmetric for super::Backend {
         use Curve::*;
         match curve {
             NistP256 | NistP384 | NistP521 | Ed25519 | Cv25519 |
-            BrainpoolP256 | BrainpoolP512
-                => true,
-            Unknown(_) if curve.is_brainpoolp384() // XXX
+            BrainpoolP256 | BrainpoolP384 | BrainpoolP512
                 => true,
             Unknown(_)
                 => false,
@@ -120,6 +120,51 @@ impl Asymmetric for super::Backend {
             secret.get_field("x")?.try_into()?))
     }
 
+    fn dsa_sign(x: &ProtectedMPI,
+                p: &MPI, q: &MPI, g: &MPI, _y: &MPI,
+                digest: &[u8])
+                -> Result<(MPI, MPI)>
+    {
+        let mut rng = RandomNumberGenerator::new_userspace()?;
+        let secret = Privkey::load_dsa(&p.try_into()?, &q.try_into()?,
+                                       &g.try_into()?, &x.try_into()?)?;
+        let size = q.value().len();
+        let truncated_digest = &digest[..size.min(digest.len())];
+        let sig = secret.sign(truncated_digest, "Raw", &mut rng)?;
+
+        if sig.len() != size * 2 {
+            return Err(Error::MalformedMPI(
+                format!("Expected signature with length {}, got {}",
+                        size * 2, sig.len())).into());
+        }
+
+        Ok((MPI::new(&sig[..size]), MPI::new(&sig[size..])))
+    }
+
+    fn dsa_verify(p: &MPI, q: &MPI, g: &MPI, y: &MPI,
+                  digest: &[u8],
+                  r: &MPI, s: &MPI)
+                  -> Result<bool>
+    {
+        // OpenPGP encodes R and S separately, but our cryptographic
+        // library expects them to be concatenated.
+        let size = q.value().len();
+        let mut sig = Vec::with_capacity(2 * size);
+
+        // We need to zero-pad them at the front, because the MPI
+        // encoding drops leading zero bytes.
+        fn bad(e: impl ToString) -> anyhow::Error {
+            Error::BadSignature(e.to_string()).into()
+        }
+        sig.extend_from_slice(&r.value_padded(size).map_err(bad)?);
+        sig.extend_from_slice(&s.value_padded(size).map_err(bad)?);
+
+        let pk = Pubkey::load_dsa(&p.try_into()?, &q.try_into()?,
+                                  &g.try_into()?, &y.try_into()?)?;
+        let truncated_digest = &digest[..size.min(digest.len())];
+        Ok(pk.verify(truncated_digest, &sig, "Raw")?)
+    }
+
     fn elgamal_generate_key(p_bits: usize)
                             -> Result<(MPI, MPI, MPI, ProtectedMPI)>
     {
@@ -208,27 +253,6 @@ impl KeyPair {
                 })
             },
 
-            (DSA,
-             PublicKey::DSA { p, q, g, .. },
-             mpi::SecretKeyMaterial::DSA { x }) => {
-                let secret = Privkey::load_dsa(&p.try_into()?, &q.try_into()?,
-                                               &g.try_into()?, &x.try_into()?)?;
-                let size = q.value().len();
-                let truncated_digest = &digest[..size.min(digest.len())];
-                let sig = secret.sign(truncated_digest, "Raw", &mut rng)?;
-
-                if sig.len() != size * 2 {
-                    return Err(Error::MalformedMPI(
-                        format!("Expected signature with length {}, got {}",
-                                size * 2, sig.len())).into());
-                }
-
-                Ok(mpi::Signature::DSA {
-                    r: MPI::new(&sig[..size]),
-                    s: MPI::new(&sig[size..]),
-                })
-            },
-
             (ECDSA,
              PublicKey::ECDSA { curve, .. },
              mpi::SecretKeyMaterial::ECDSA { scalar }) => {
@@ -373,11 +397,12 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(), data),
 
-            RSASign | DSA | ECDSA | EdDSA =>
+            RSASign | DSA | ECDSA | EdDSA | Ed25519 | Ed448 =>
                 Err(Error::InvalidOperation(
                     format!("{} is not an encryption algorithm", self.pk_algo())
                 ).into()),
 
+            X25519 | X448 |
             Private(_) | Unknown(_) =>
                 Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
         }
@@ -399,23 +424,7 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                 pk.verify(digest, s.value(),
                           &format!("PKCS1v15(Raw,{})", hash_algo.botan_name()?))?
             },
-            (PublicKey::DSA { y, q, p, g }, Signature::DSA { s, r }) => {
-                // OpenPGP encodes R and S separately, but our
-                // cryptographic library expects them to be
-                // concatenated.
-                let size = q.value().len();
-                let mut sig = Vec::with_capacity(2 * size);
 
-                // We need to zero-pad them at the front, because
-                // the MPI encoding drops leading zero bytes.
-                sig.extend_from_slice(&r.value_padded(size).map_err(bad)?);
-                sig.extend_from_slice(&s.value_padded(size).map_err(bad)?);
-
-                let pk = Pubkey::load_dsa(&p.try_into()?, &q.try_into()?,
-                                          &g.try_into()?, &y.try_into()?)?;
-                let truncated_digest = &digest[..size.min(digest.len())];
-                pk.verify(truncated_digest, &sig, "Raw").unwrap()
-            },
             (PublicKey::ECDSA { curve, q }, Signature::ECDSA { s, r }) =>
             {
                 // OpenPGP encodes R and S separately, but our
@@ -451,12 +460,11 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 impl<R> Key4<SecretParts, R>
     where R: key::KeyRole,
 {
-    /// Creates a new OpenPGP public key packet for an existing RSA key.
+    /// Creates a new OpenPGP secret key packet for an existing RSA key.
     ///
-    /// The RSA key will use public exponent `e` and modulo `n`. The key will
-    /// have it's creation date set to `ctime` or the current time if `None`
-    /// is given.
-    #[allow(clippy::many_single_char_names)]
+    /// The RSA key will use the secret exponent `d`, derived from the
+    /// secret primes `p` and `q`.  The key will have its creation
+    /// date set to `ctime` or the current time if `None` is given.
     pub fn import_secret_rsa<T>(d: &[u8], p: &[u8], q: &[u8], ctime: T)
         -> Result<Self> where T: Into<Option<SystemTime>>
     {
@@ -504,8 +512,7 @@ impl<R> Key4<SecretParts, R>
         let mut rng = RandomNumberGenerator::new_userspace()?;
         let hash = crate::crypto::ecdh::default_ecdh_kdf_hash(&curve);
         let sym = crate::crypto::ecdh::default_ecdh_kek_cipher(&curve);
-        let field_sz_bits = curve.bits()
-            .ok_or_else(|| Error::UnsupportedEllipticCurve(curve.clone()))?;
+        let field_sz_bits = curve.bits()?;
 
         match (curve, for_signing) {
             (Curve::Ed25519, true) =>
@@ -601,7 +608,7 @@ impl Curve {
             NistP384 => Ok("secp384r1"),
             NistP521 => Ok("secp521r1"),
             BrainpoolP256 => Ok("brainpool256r1"),
-            Unknown(_) if self.is_brainpoolp384() => Ok("brainpool384r1"),
+            BrainpoolP384 => Ok("brainpool384r1"),
             BrainpoolP512 => Ok("brainpool512r1"),
             Ed25519 | // Handled differently.
             Cv25519 | // Handled differently.
