@@ -2,14 +2,14 @@
 use std::cmp::{self, Ordering};
 
 use crate::{Error, Result};
-use crate::crypto::aead::{Aead, CipherOp};
-use crate::crypto::mem::secure_cmp;
+use crate::crypto::aead::{Context, CipherOp};
+use crate::crypto::mem::{Protected, secure_cmp};
 use crate::seal;
 use crate::types::{AEADAlgorithm, SymmetricAlgorithm};
 
 use cipher::generic_array::{GenericArray, ArrayLength};
-use cipher::generic_array::typenum::{U12, U16, U128, U192, U256};
-use cipher::{BlockCipher, BlockEncrypt, BlockSizeUser, KeyInit, Unsigned};
+use cipher::generic_array::typenum::{U12, U15, U16, U128, U192, U256};
+use cipher::{BlockCipher, BlockDecrypt, BlockEncrypt, BlockSizeUser, KeyInit, Unsigned};
 use eax::online::{Eax, Encrypt, Decrypt};
 use win_crypto_ng::symmetric::{BlockCipherKey, Aes};
 
@@ -38,17 +38,60 @@ impl<T, N: ArrayLength<T>> GenericArrayExt<T, N> for GenericArray<T, N> {
     const LEN: usize = N::USIZE;
 }
 
-impl AEADAlgorithm {
-    pub(crate) fn context(
-        &self,
+
+impl crate::crypto::backend::interface::Aead for super::Backend {
+    fn supports_algo(algo: AEADAlgorithm) -> bool {
+        use self::AEADAlgorithm::*;
+        match algo {
+            EAX => true,
+            OCB => true,
+            GCM => true,
+            Private(_) | Unknown(_)
+                => false,
+        }
+    }
+
+    fn supports_algo_with_symmetric(algo: AEADAlgorithm,
+                                    symm: SymmetricAlgorithm)
+                                    -> bool
+    {
+        match algo {
+            AEADAlgorithm::EAX => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::OCB => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::GCM => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::Private(_) |
+            AEADAlgorithm::Unknown(_) => false,
+        }
+    }
+
+    fn context(
+        algo: AEADAlgorithm,
         sym_algo: SymmetricAlgorithm,
-        key: &[u8],
+        key: &Protected,
         aad: &[u8],
         nonce: &[u8],
         op: CipherOp,
-    ) -> Result<Box<dyn Aead>> {
-
-        match self {
+    ) -> Result<Box<dyn Context>> {
+        #[allow(deprecated)]
+        match algo {
             AEADAlgorithm::EAX => match sym_algo {
                 SymmetricAlgorithm::AES128 => match op {
                     CipherOp::Encrypt => {
@@ -107,8 +150,41 @@ impl AEADAlgorithm {
                 _ => Err(Error::UnsupportedSymmetricAlgorithm(sym_algo).into()),
             },
 
-            AEADAlgorithm::OCB =>
-                Err(Error::UnsupportedAEADAlgorithm(*self).into()),
+            AEADAlgorithm::OCB => {
+                use ocb3::{Ocb3, Nonce};
+                match sym_algo {
+                    SymmetricAlgorithm::AES128 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<BlockCipherKey<Aes, U128>, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::AES192 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<BlockCipherKey<Aes, U192>, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::AES256 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<BlockCipherKey<Aes, U256>, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    | SymmetricAlgorithm::IDEA
+                    | SymmetricAlgorithm::TripleDES
+                    | SymmetricAlgorithm::CAST5
+                    | SymmetricAlgorithm::Blowfish
+                    | SymmetricAlgorithm::Twofish
+                    | SymmetricAlgorithm::Camellia128
+                    | SymmetricAlgorithm::Camellia192
+                    | SymmetricAlgorithm::Camellia256
+                    | SymmetricAlgorithm::Private(_)
+                    | SymmetricAlgorithm::Unknown(_)
+                    | SymmetricAlgorithm::Unencrypted =>
+                        Err(Error::UnsupportedSymmetricAlgorithm(sym_algo).into()),
+                }
+            },
 
             AEADAlgorithm::GCM => {
                 use aes_gcm::{AesGcm, Nonce};
@@ -147,7 +223,7 @@ impl AEADAlgorithm {
             },
 
             AEADAlgorithm::Private(_) | AEADAlgorithm::Unknown(_) =>
-                Err(Error::UnsupportedAEADAlgorithm(*self).into()),
+                Err(Error::UnsupportedAEADAlgorithm(algo).into()),
         }
     }
 }
@@ -157,7 +233,7 @@ type TagLen = U16;
 macro_rules! impl_aead {
     ($($type: ty),*) => {
         $(
-        impl Aead for Eax<$type, Encrypt, TagLen> {
+        impl Context for Eax<$type, Encrypt, TagLen> {
             fn digest_size(&self) -> usize {
                 TagLen::USIZE
             }
@@ -178,7 +254,7 @@ macro_rules! impl_aead {
         impl seal::Sealed for Eax<$type, Encrypt> {}
         )*
         $(
-        impl Aead for Eax<$type, Decrypt> {
+        impl Context for Eax<$type, Decrypt> {
             fn digest_size(&self) -> usize {
                 TagLen::USIZE
             }
@@ -213,13 +289,65 @@ macro_rules! impl_aead {
 
 impl_aead!(BlockCipherKey<Aes, U128>, BlockCipherKey<Aes, U192>, BlockCipherKey<Aes, U256>);
 
+struct Ocb<Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt> {
+    cipher: ocb3::Ocb3<Cipher, U15>,
+    nonce: GenericArray<u8, U15>,
+    aad: Vec<u8>,
+}
+
+impl<Cipher> Context for Ocb<Cipher>
+where
+    Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
+{
+    fn digest_size(&self) -> usize {
+        TagLen::USIZE
+    }
+
+    fn encrypt_seal(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()> {
+        debug_assert_eq!(dst.len(), src.len() + self.digest_size());
+        use ocb3::AeadInPlace;
+
+        let len = cmp::min(dst.len(), src.len());
+        dst[..len].copy_from_slice(&src[..len]);
+        let tag =
+            self.cipher.encrypt_in_place_detached(&self.nonce, &self.aad,
+                                                  &mut dst[..len])?;
+        debug_assert_eq!(dst[len..].len(), tag.len());
+        let tag_len = cmp::min(dst[len..].len(), tag.len());
+        dst[len..len + tag_len].copy_from_slice(&tag[..tag_len]);
+        Ok(())
+    }
+
+    fn decrypt_verify(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()> {
+        debug_assert_eq!(dst.len() + self.digest_size(), src.len());
+        use ocb3::AeadInPlace;
+
+        // Split src into ciphertext and digest.
+        let len = src.len().saturating_sub(self.digest_size());
+        let digest = &src[len..];
+        let src = &src[..len];
+
+        debug_assert_eq!(dst.len(), src.len());
+        let len = core::cmp::min(dst.len(), src.len());
+        dst[..len].copy_from_slice(&src[..len]);
+        self.cipher.decrypt_in_place_detached(&self.nonce, &self.aad, dst,
+                                              digest.try_into()?)?;
+        Ok(())
+    }
+}
+
+impl<'a, Cipher> seal::Sealed for Ocb<Cipher>
+where
+    Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
+{}
+
 struct Gcm<Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockEncrypt> {
     cipher: aes_gcm::AesGcm<Cipher, U12>,
     nonce: GenericArray<u8, U12>,
     aad: Vec<u8>,
 }
 
-impl<Cipher> Aead for Gcm<Cipher>
+impl<Cipher> Context for Gcm<Cipher>
 where
     Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockEncrypt,
 {
