@@ -5,7 +5,7 @@ use crate::Result;
 use crate::types::{HashAlgorithm};
 
 #[derive(Clone)]
-struct NullHasher(HashAlgorithm);
+struct NullHasher();
 
 impl io::Write for NullHasher {
     fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
@@ -17,23 +17,6 @@ impl io::Write for NullHasher {
 }
 
 impl Digest for NullHasher {
-    fn algo(&self) -> HashAlgorithm {
-        self.0
-    }
-
-    fn digest_size(&self) -> usize {
-        match self.0 {
-            HashAlgorithm::SHA1 => 20,
-            HashAlgorithm::SHA224 => 28,
-            HashAlgorithm::SHA256 => 32,
-            HashAlgorithm::SHA384 => 48,
-            HashAlgorithm::SHA512 => 64,
-            HashAlgorithm::RipeMD => 20,
-            HashAlgorithm::MD5 => 16,
-            _ => 32, // Made up.
-        }
-    }
-
     fn update(&mut self, data: &[u8]) {
         // Nop.
     }
@@ -47,7 +30,7 @@ impl Digest for NullHasher {
 impl HashAlgorithm {
     /// Whether Sequoia supports this algorithm.
     pub fn is_supported(self) -> bool {
-        true
+        self.digest_size().is_ok()
     }
 
     /// Creates a new hash context for this algorithm.
@@ -60,6 +43,6 @@ impl HashAlgorithm {
     ///
     ///   [`HashAlgorithm::is_supported`]: HashAlgorithm::is_supported()
     pub(crate) fn new_hasher(self) -> Result<Box<dyn Digest>> {
-        Ok(Box::new(NullHasher(self)))
+        Ok(Box::new(NullHasher()))
     }
 }
