@@ -4,7 +4,18 @@
 //! [`Decryptor`]: crate::crypto::Decryptor
 //! [`KeyPair`]: crate::crypto::KeyPair
 
-use nettle::{curve25519, ecc, ecdh, ecdsa, ed25519, dsa, rsa, random::Yarrow};
+use nettle::{
+    curve25519,
+    curve448,
+    dsa,
+    ecc,
+    ecdh,
+    ecdsa,
+    ed25519,
+    ed448,
+    rsa,
+    random::Yarrow,
+};
 
 use crate::{Error, Result};
 
@@ -12,7 +23,10 @@ use crate::packet::{key, Key};
 use crate::crypto::asymmetric::KeyPair;
 use crate::crypto::backend::interface::Asymmetric;
 use crate::crypto::mpi::{self, MPI, ProtectedMPI, PublicKey};
-use crate::crypto::SessionKey;
+use crate::crypto::{
+    SessionKey,
+    mem::Protected,
+};
 use crate::types::{Curve, HashAlgorithm};
 
 impl Asymmetric for super::Backend {
@@ -20,8 +34,11 @@ impl Asymmetric for super::Backend {
         use PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match algo {
+            X25519 | Ed25519 |
             RSAEncryptSign | RSAEncrypt | RSASign | DSA | ECDH | ECDSA | EdDSA
                 => true,
+            X448 | Ed448
+                => curve448::IS_SUPPORTED,
             ElGamalEncrypt | ElGamalEncryptSign | Private(_) | Unknown(_)
                 => false,
         }
@@ -32,7 +49,7 @@ impl Asymmetric for super::Backend {
         match curve {
             NistP256 | NistP384 | NistP521 | Ed25519 | Cv25519
                 => true,
-            BrainpoolP256 | BrainpoolP512 | Unknown(_)
+            BrainpoolP256 | BrainpoolP384 | BrainpoolP512 | Unknown(_)
                 => false,
         }
     }
@@ -61,6 +78,42 @@ impl Asymmetric for super::Backend {
         Ok(s)
     }
 
+    fn x448_generate_key() -> Result<(Protected, [u8; 56])> {
+        debug_assert_eq!(curve448::CURVE448_SIZE, 56);
+        if ! curve448::IS_SUPPORTED {
+            return Err(Error::UnsupportedPublicKeyAlgorithm(
+                PublicKeyAlgorithm::Ed448).into());
+        }
+        let mut rng = Yarrow::default();
+        let secret = curve448::private_key(&mut rng);
+        let mut public = [0; 56];
+        curve448::mul_g(&mut public, &secret)?;
+        Ok((secret.into(), public))
+    }
+
+    fn x448_derive_public(secret: &Protected) -> Result<[u8; 56]> {
+        debug_assert_eq!(curve448::CURVE448_SIZE, 56);
+        if ! curve448::IS_SUPPORTED {
+            return Err(Error::UnsupportedPublicKeyAlgorithm(
+                PublicKeyAlgorithm::Ed448).into());
+        }
+        let mut public = [0; 56];
+        curve448::mul_g(&mut public, secret)?;
+        Ok(public)
+    }
+
+    fn x448_shared_point(secret: &Protected, public: &[u8; 56])
+                           -> Result<Protected> {
+        debug_assert_eq!(curve448::CURVE448_SIZE, 56);
+        if ! curve448::IS_SUPPORTED {
+            return Err(Error::UnsupportedPublicKeyAlgorithm(
+                PublicKeyAlgorithm::Ed448).into());
+        }
+        let mut s: Protected = vec![0; 56].into();
+        curve448::mul(&mut s, secret, public)?;
+        Ok(s)
+    }
+
     fn ed25519_generate_key() -> Result<(Protected, [u8; 32])> {
         debug_assert_eq!(ed25519::ED25519_KEY_SIZE, 32);
         let mut rng = Yarrow::default();
@@ -94,6 +147,39 @@ impl Asymmetric for super::Backend {
         Ok(ed25519::verify(public, digest, signature)?)
     }
 
+    fn ed448_generate_key() -> Result<(Protected, [u8; 57])> {
+        debug_assert_eq!(ed448::ED448_KEY_SIZE, 57);
+        let mut rng = Yarrow::default();
+        let mut public = [0; 57];
+        let secret: Protected =
+            ed448::private_key(&mut rng).into();
+        ed448::public_key(&mut public, &secret)?;
+        Ok((secret, public))
+    }
+
+    fn ed448_derive_public(secret: &Protected) -> Result<[u8; 57]> {
+        debug_assert_eq!(ed448::ED448_KEY_SIZE, 57);
+        let mut public = [0; 57];
+        ed448::public_key(&mut public, secret)?;
+        Ok(public)
+    }
+
+    fn ed448_sign(secret: &Protected, public: &[u8; 57], digest: &[u8])
+                    -> Result<[u8; 114]> {
+        debug_assert_eq!(ed448::ED448_KEY_SIZE, 57);
+        debug_assert_eq!(ed448::ED448_SIGNATURE_SIZE, 114);
+        let mut sig = [0u8; 114];
+        ed448::sign(public, secret, digest, &mut sig)?;
+        Ok(sig)
+    }
+
+    fn ed448_verify(public: &[u8; 57], digest: &[u8], signature: &[u8; 114])
+                      -> Result<bool> {
+        debug_assert_eq!(ed448::ED448_KEY_SIZE, 57);
+        debug_assert_eq!(ed448::ED448_SIGNATURE_SIZE, 114);
+        Ok(ed448::verify(public, digest, signature)?)
+    }
+
     fn dsa_generate_key(p_bits: usize)
                         -> Result<(MPI, MPI, MPI, MPI, ProtectedMPI)>
     {
@@ -106,6 +192,31 @@ impl Asymmetric for super::Backend {
         Ok((p.into(), q.into(), g.into(), y.as_bytes().into(),
             x.as_bytes().into()))
     }
+
+    fn dsa_sign(x: &ProtectedMPI,
+                p: &MPI, q: &MPI, g: &MPI, _y: &MPI,
+                digest: &[u8])
+                -> Result<(MPI, MPI)>
+    {
+        let mut rng = Yarrow::default();
+        let params = dsa::Params::new(p.value(), q.value(), g.value());
+        let secret = dsa::PrivateKey::new(x.value());
+
+        let sig = dsa::sign(&params, &secret, digest, &mut rng)?;
+
+        Ok((MPI::new(&sig.r()), MPI::new(&sig.s())))
+    }
+
+    fn dsa_verify(p: &MPI, q: &MPI, g: &MPI, y: &MPI,
+                  digest: &[u8],
+                  r: &MPI, s: &MPI)
+                  -> Result<bool>
+    {
+        let key = dsa::PublicKey::new(y.value());
+        let params = dsa::Params::new(p.value(), q.value(), g.value());
+        let signature = dsa::Signature::new(r.value(), s.value());
+        Ok(dsa::verify(&params, &key, digest, &signature))
+    }
 }
 
 impl KeyPair {
@@ -135,12 +246,12 @@ impl KeyPair {
                 // The signature has the length of the modulus.
                 let mut sig = vec![0u8; n.value().len()];
 
-                // As described in [Section 5.2.2 and 5.2.3 of RFC 4880],
+                // As described in [Section 5.2.2 and 5.2.3 of RFC 9580],
                 // to verify the signature, we need to encode the
                 // signature data in a PKCS1-v1.5 packet.
                 //
-                //   [Section 5.2.2 and 5.2.3 of RFC 4880]:
-                //   https://tools.ietf.org/html/rfc4880#section-5.2.2
+                //   [Section 5.2.2 and 5.2.3 of RFC 9580]:
+                //   https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.2
                 rsa::sign_digest_pkcs1(&public, &secret, digest,
                                        hash_algo.oid()?,
                                        &mut rng, &mut sig)?;
@@ -150,20 +261,6 @@ impl KeyPair {
                 })
             },
 
-            (DSA,
-             &PublicKey::DSA { ref p, ref q, ref g, .. },
-             &mpi::SecretKeyMaterial::DSA { ref x }) => {
-                let params = dsa::Params::new(p.value(), q.value(), g.value());
-                let secret = dsa::PrivateKey::new(x.value());
-
-                let sig = dsa::sign(&params, &secret, digest, &mut rng)?;
-
-                Ok(mpi::Signature::DSA {
-                    r: MPI::new(&sig.r()),
-                    s: MPI::new(&sig.s()),
-                })
-            },
-
             (ECDSA,
              &PublicKey::ECDSA { ref curve, .. },
              &mpi::SecretKeyMaterial::ECDSA { ref scalar }) => {
@@ -228,9 +325,11 @@ impl KeyPair {
 
             (PublicKey::ElGamal{ .. },
              mpi::SecretKeyMaterial::ElGamal{ .. },
-             mpi::Ciphertext::ElGamal{ .. }) =>
+             mpi::Ciphertext::ElGamal{ .. }) => {
+                #[allow(deprecated)]
                 return Err(
-                    Error::UnsupportedPublicKeyAlgorithm(ElGamalEncrypt).into()),
+                    Error::UnsupportedPublicKeyAlgorithm(ElGamalEncrypt).into());
+            },
 
             (PublicKey::ECDH{ .. },
              mpi::SecretKeyMaterial::ECDH { .. },
@@ -287,11 +386,13 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(),
                                                  data),
 
-            RSASign | DSA | ECDSA | EdDSA =>
+            RSASign | DSA | ECDSA | EdDSA | Ed25519 | Ed448 =>
                 Err(Error::InvalidOperation(
                     format!("{} is not an encryption algorithm", self.pk_algo())
                 ).into()),
 
+            X25519 | // Handled in common code.
+            X448 | // Handled in common code.
             ElGamalEncrypt | ElGamalEncryptSign |
             Private(_) | Unknown(_) =>
                 Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
@@ -308,22 +409,15 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
             (PublicKey::RSA { e, n }, Signature::RSA { s }) => {
                 let key = rsa::PublicKey::new(n.value(), e.value())?;
 
-                // As described in [Section 5.2.2 and 5.2.3 of RFC 4880],
+                // As described in [Section 5.2.2 and 5.2.3 of RFC 9580],
                 // to verify the signature, we need to encode the
                 // signature data in a PKCS1-v1.5 packet.
                 //
-                //   [Section 5.2.2 and 5.2.3 of RFC 4880]:
-                //   https://tools.ietf.org/html/rfc4880#section-5.2.2
+                //   [Section 5.2.2 and 5.2.3 of RFC 9580]:
+                //   https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.2
                 rsa::verify_digest_pkcs1(&key, digest, hash_algo.oid()?,
                                          s.value())?
             },
-            (PublicKey::DSA { y, p, q, g }, Signature::DSA { s, r }) => {
-                let key = dsa::PublicKey::new(y.value());
-                let params = dsa::Params::new(p.value(), q.value(), g.value());
-                let signature = dsa::Signature::new(r.value(), s.value());
-
-                dsa::verify(&params, &key, digest, &signature)
-            },
             (PublicKey::ECDSA { curve, q }, Signature::ECDSA { s, r }) =>
             {
                 let (x, y) = q.decode_point(curve)?;
@@ -352,18 +446,17 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 }
 
 use std::time::SystemTime;
-use crate::crypto::mem::Protected;
 use crate::packet::key::{Key4, SecretParts};
 use crate::types::PublicKeyAlgorithm;
 
 impl<R> Key4<SecretParts, R>
     where R: key::KeyRole,
 {
-    /// Creates a new OpenPGP public key packet for an existing RSA key.
+    /// Creates a new OpenPGP secret key packet for an existing RSA key.
     ///
-    /// The RSA key will use public exponent `e` and modulo `n`. The key will
-    /// have it's creation date set to `ctime` or the current time if `None`
-    /// is given.
+    /// The RSA key will use the secret exponent `d`, derived from the
+    /// secret primes `p` and `q`.  The key will have its creation
+    /// date set to `ctime` or the current time if `None` is given.
     pub fn import_secret_rsa<T>(d: &[u8], p: &[u8], q: &[u8], ctime: T)
         -> Result<Self> where T: Into<Option<SystemTime>>
     {
