@@ -2,7 +2,8 @@
 
 use crate::{Error, Result};
 
-use crate::crypto::aead::{Aead, CipherOp};
+use crate::crypto::aead::{Context, CipherOp};
+use crate::crypto::mem::Protected;
 use crate::types::{AEADAlgorithm, SymmetricAlgorithm};
 
 use openssl::cipher::Cipher;
@@ -13,7 +14,7 @@ struct OpenSslContext {
     digest_size: usize,
 }
 
-impl Aead for OpenSslContext {
+impl Context for OpenSslContext {
     fn encrypt_seal(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()> {
         debug_assert_eq!(dst.len(), src.len() + self.digest_size());
 
@@ -53,16 +54,76 @@ impl Aead for OpenSslContext {
 
 impl crate::seal::Sealed for OpenSslContext {}
 
-impl AEADAlgorithm {
-    pub(crate) fn context(
-        &self,
+
+impl crate::crypto::backend::interface::Aead for super::Backend {
+    fn supports_algo(algo: AEADAlgorithm) -> bool {
+        // First, check whether support is compiled in or not.
+        (match algo {
+            AEADAlgorithm::EAX => false,
+            AEADAlgorithm::OCB => cfg!(not(osslconf = "OPENSSL_NO_OCB")),
+            AEADAlgorithm::GCM => true,
+            AEADAlgorithm::Private(_) |
+            AEADAlgorithm::Unknown(_) => false,
+        }) && {
+            // Then, check whether it has been disabled at runtime,
+            // for example by putting OpenSSL into FIPS mode.
+            use std::sync::OnceLock;
+
+            static MODES: [OnceLock<bool>; 3] = [const { OnceLock::new() }; 3];
+            MODES.get(u8::from(algo) as usize - 1)
+                .map(|cell| cell.get_or_init(|| {
+                    let symm = SymmetricAlgorithm::AES128;
+                    let key = Protected::from([0; 16]);
+                    let nonce = &key[..algo.nonce_size().unwrap_or(16)];
+
+                    Self::context(algo, symm, &key, &[], nonce,
+                                  crate::crypto::aead::CipherOp::Encrypt)
+                        .is_ok()
+                }))
+                .cloned()
+                .unwrap_or(false)
+        }
+    }
+
+    fn supports_algo_with_symmetric(algo: AEADAlgorithm,
+                                    symm: SymmetricAlgorithm)
+                                    -> bool
+    {
+        match algo {
+            AEADAlgorithm::EAX => false,
+
+            AEADAlgorithm::OCB => match symm {
+                // OpenSSL supports OCB only with AES
+                // see: https://wiki.openssl.org/index.php/OCB
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::GCM => match symm {
+                // OpenSSL supports GCM only with AES
+                // see: https://wiki.openssl.org/index.php/GCM
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::Private(_) |
+            AEADAlgorithm::Unknown(_) => false,
+        }
+    }
+
+    fn context(
+        algo: AEADAlgorithm,
         sym_algo: SymmetricAlgorithm,
-        key: &[u8],
+        key: &Protected,
         aad: &[u8],
         nonce: &[u8],
         op: CipherOp,
-    ) -> Result<Box<dyn Aead>> {
-        match self {
+    ) -> Result<Box<dyn Context>> {
+        match algo {
             #[cfg(not(osslconf = "OPENSSL_NO_OCB"))]
             AEADAlgorithm::OCB => {
                 let cipher = match sym_algo {
@@ -81,7 +142,7 @@ impl AEADAlgorithm {
                 }
                 // We have to set the IV length before supplying the
                 // IV.  Otherwise, it will be silently truncated.
-                ctx.set_iv_length(self.nonce_size()?)?;
+                ctx.set_iv_length(algo.nonce_size()?)?;
                 match op {
                     CipherOp::Encrypt =>
                         ctx.encrypt_init(None, None, Some(nonce))?,
@@ -93,9 +154,10 @@ impl AEADAlgorithm {
                 ctx.cipher_update(aad, None)?;
                 Ok(Box::new(OpenSslContext {
                     ctx,
-                    digest_size: self.digest_size()?,
+                    digest_size: algo.digest_size()?,
                 }))
             },
+
             AEADAlgorithm::GCM => {
                 let cipher = match sym_algo {
                     SymmetricAlgorithm::AES128 => Cipher::aes_128_gcm(),
@@ -113,7 +175,7 @@ impl AEADAlgorithm {
                 }
                 // We have to set the IV length before supplying the
                 // IV.  Otherwise, it will be silently truncated.
-                ctx.set_iv_length(self.nonce_size()?)?;
+                ctx.set_iv_length(algo.nonce_size()?)?;
                 match op {
                     CipherOp::Encrypt =>
                         ctx.encrypt_init(None, None, Some(nonce))?,
@@ -125,10 +187,11 @@ impl AEADAlgorithm {
                 ctx.cipher_update(aad, None)?;
                 Ok(Box::new(OpenSslContext {
                     ctx,
-                    digest_size: self.digest_size()?,
+                    digest_size: algo.digest_size()?,
                 }))
             },
-            _ => Err(Error::UnsupportedAEADAlgorithm(*self).into()),
+
+            _ => Err(Error::UnsupportedAEADAlgorithm(algo).into()),
         }
     }
 }
