@@ -9,8 +9,8 @@ use eax::online::{Eax, Encrypt, Decrypt};
 use cipher::generic_array::GenericArray;
 
 use crate::{Error, Result};
-use crate::crypto::aead::{Aead, CipherOp};
-use crate::crypto::mem::secure_cmp;
+use crate::crypto::aead::{Context, CipherOp};
+use crate::crypto::mem::{Protected, secure_cmp};
 use crate::seal;
 use crate::types::{AEADAlgorithm, SymmetricAlgorithm};
 
@@ -24,7 +24,7 @@ const DANGER_DISABLE_AUTHENTICATION: bool = false;
 
 type TagLen = U16;
 
-impl<Cipher> Aead for Eax<Cipher, Encrypt, TagLen>
+impl<Cipher> Context for Eax<Cipher, Encrypt, TagLen>
 where
     Cipher: BlockCipher<BlockSize = U16> + BlockEncrypt + Clone + KeyInit,
 {
@@ -47,7 +47,7 @@ where
     }
 }
 
-impl<Cipher> Aead for Eax<Cipher, Decrypt, TagLen>
+impl<Cipher> Context for Eax<Cipher, Decrypt, TagLen>
 where
     Cipher: BlockCipher<BlockSize = U16> + BlockEncrypt + Clone + KeyInit,
 {
@@ -93,7 +93,7 @@ struct Gcm<Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockEncrypt>
     aad: Vec<u8>,
 }
 
-impl<Cipher> Aead for Gcm<Cipher>
+impl<Cipher> Context for Gcm<Cipher>
 where
     Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockEncrypt,
 {
@@ -145,7 +145,7 @@ struct Ocb<Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt +
     aad: Vec<u8>,
 }
 
-impl<Cipher> Aead for Ocb<Cipher>
+impl<Cipher> Context for Ocb<Cipher>
 where
     Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
 {
@@ -191,17 +191,68 @@ where
     Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
 {}
 
-impl AEADAlgorithm {
-    pub(crate) fn context(
-        &self,
+
+impl crate::crypto::backend::interface::Aead for super::Backend {
+    fn supports_algo(algo: AEADAlgorithm) -> bool {
+        use self::AEADAlgorithm::*;
+        match algo {
+            EAX => true,
+            OCB => true,
+            GCM => true,
+            Private(_) | Unknown(_)
+                => false,
+        }
+    }
+
+    fn supports_algo_with_symmetric(algo: AEADAlgorithm,
+                                    symm: SymmetricAlgorithm)
+                                    -> bool
+    {
+        match algo {
+            AEADAlgorithm::EAX => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 |
+                SymmetricAlgorithm::Twofish |
+                SymmetricAlgorithm::Camellia128 |
+                SymmetricAlgorithm::Camellia192 |
+                SymmetricAlgorithm::Camellia256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::OCB => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 |
+                SymmetricAlgorithm::Twofish |
+                SymmetricAlgorithm::Camellia128 |
+                SymmetricAlgorithm::Camellia192 |
+                SymmetricAlgorithm::Camellia256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::GCM => match symm {
+                SymmetricAlgorithm::AES128 |
+                SymmetricAlgorithm::AES192 |
+                SymmetricAlgorithm::AES256 => true,
+                _ => false,
+            },
+
+            AEADAlgorithm::Private(_) |
+            AEADAlgorithm::Unknown(_) => false,
+        }
+    }
+
+    fn context(
+        algo: AEADAlgorithm,
         sym_algo: SymmetricAlgorithm,
-        key: &[u8],
+        key: &Protected,
         aad: &[u8],
         nonce: &[u8],
         op: CipherOp,
-    ) -> Result<Box<dyn Aead>> {
+    ) -> Result<Box<dyn Context>> {
         #[allow(deprecated)]
-        match self {
+        match algo {
             AEADAlgorithm::EAX => match sym_algo {
                 SymmetricAlgorithm::AES128 => match op {
                     CipherOp::Encrypt => {
@@ -257,6 +308,24 @@ impl AEADAlgorithm {
                         Ok(Box::new(ctx))
                     },
                 },
+                SymmetricAlgorithm::Twofish => match op {
+                    CipherOp::Encrypt => {
+                        let mut ctx =
+                            Eax::<twofish::Twofish, Encrypt>::with_key_and_nonce(
+                                GenericArray::try_from_slice(key)?,
+                                GenericArray::try_from_slice(nonce)?);
+                        ctx.update_assoc(aad);
+                        Ok(Box::new(ctx))
+                    },
+                    CipherOp::Decrypt => {
+                        let mut ctx =
+                            Eax::<twofish::Twofish, Decrypt>::with_key_and_nonce(
+                                GenericArray::try_from_slice(key)?,
+                                GenericArray::try_from_slice(nonce)?);
+                        ctx.update_assoc(aad);
+                        Ok(Box::new(ctx))
+                    },
+                },
                 SymmetricAlgorithm::Camellia128 => match op {
                     CipherOp::Encrypt => {
                         let mut ctx =
@@ -315,7 +384,6 @@ impl AEADAlgorithm {
                 | SymmetricAlgorithm::TripleDES
                 | SymmetricAlgorithm::CAST5
                 | SymmetricAlgorithm::Blowfish
-                | SymmetricAlgorithm::Twofish
                 | SymmetricAlgorithm::Private(_)
                 | SymmetricAlgorithm::Unknown(_)
                 | SymmetricAlgorithm::Unencrypted =>
@@ -343,6 +411,12 @@ impl AEADAlgorithm {
                             Ocb3::<aes::Aes256, U15>::new_from_slice(key)?;
                         Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
                     },
+                    SymmetricAlgorithm::Twofish => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<twofish::Twofish, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
                     SymmetricAlgorithm::Camellia128 => {
                         let nonce = Nonce::try_from_slice(nonce)?;
                         let cipher =
@@ -365,7 +439,6 @@ impl AEADAlgorithm {
                     | SymmetricAlgorithm::TripleDES
                     | SymmetricAlgorithm::CAST5
                     | SymmetricAlgorithm::Blowfish
-                    | SymmetricAlgorithm::Twofish
                     | SymmetricAlgorithm::Private(_)
                     | SymmetricAlgorithm::Unknown(_)
                     | SymmetricAlgorithm::Unencrypted =>
@@ -410,7 +483,7 @@ impl AEADAlgorithm {
             },
 
             AEADAlgorithm::Private(_) | AEADAlgorithm::Unknown(_) =>
-                Err(Error::UnsupportedAEADAlgorithm(*self).into()),
+                Err(Error::UnsupportedAEADAlgorithm(algo).into()),
         }
     }
 }
