@@ -55,10 +55,12 @@ impl Asymmetric for super::Backend {
         use PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match algo {
+            X25519 | Ed25519 |
             RSAEncryptSign | RSAEncrypt | RSASign | ECDH | EdDSA | ECDSA
                 => true,
             DSA
                 => true,
+            X448 | Ed448 |
             ElGamalEncrypt | ElGamalEncryptSign | Private(_) | Unknown(_)
                 => false,
         }
@@ -71,7 +73,7 @@ impl Asymmetric for super::Backend {
                 => true,
             Ed25519 | Cv25519
                 => true,
-            BrainpoolP256 | BrainpoolP512 | Unknown(_)
+            BrainpoolP256 | BrainpoolP384 | BrainpoolP512 | Unknown(_)
                 => false,
         }
     }
@@ -86,7 +88,7 @@ impl Asymmetric for super::Backend {
         let secret = StaticSecret::random_from_rng(&mut OsRng);
         let public = PublicKey::from(&secret);
         let mut secret_bytes = secret.to_bytes();
-        let secret: Protected = secret_bytes.as_ref().into();
+        let secret: Protected = secret_bytes.into();
         unsafe {
             memsec::memzero(secret_bytes.as_mut_ptr(), secret_bytes.len());
         }
@@ -137,10 +139,6 @@ impl Asymmetric for super::Backend {
         let public = VerifyingKey::from_bytes(public).map_err(|e| {
             Error::InvalidKey(e.to_string())
         })?;
-        let signature = signature.as_ref().try_into().map_err(|e: std::array::TryFromSliceError| {
-            Error::InvalidArgument(e.to_string())
-        })?;
-
         let signature = Signature::from_bytes(signature);
         Ok(public.verify(digest, &signature).is_ok())
     }
@@ -167,6 +165,36 @@ impl Asymmetric for super::Backend {
 
         Ok((p, q, g, public.y().into(), secret.x().into()))
     }
+
+    fn dsa_sign(x: &ProtectedMPI,
+                p: &MPI, q: &MPI, g: &MPI, y: &MPI,
+                digest: &[u8])
+                -> Result<(MPI, MPI)>
+    {
+        use dsa::signature::hazmat::PrehashSigner;
+        let c = dsa::Components::from_components(
+            p.into(), q.into(), g.into())?;
+        let public =
+            dsa::VerifyingKey::from_components(c, y.into())?;
+        let secret =
+            dsa::SigningKey::from_components(public, x.into())?;
+        let sig = secret.sign_prehash(digest)?;
+        Ok((sig.r().into(), sig.s().into()))
+    }
+
+    fn dsa_verify(p: &MPI, q: &MPI, g: &MPI, y: &MPI,
+                  digest: &[u8],
+                  r: &MPI, s: &MPI)
+                  -> Result<bool>
+    {
+        use dsa::signature::hazmat::PrehashVerifier;
+        let c = dsa::Components::from_components(
+            p.into(), q.into(), g.into())?;
+        let public = dsa::VerifyingKey::from_components(c, y.into())?;
+        let sig = dsa::Signature::from_components(r.into(), s.into())?;
+        public.verify_prehash(digest, &sig)?;
+        Ok(true)
+    }
 }
 
 impl From<&BigUint> for ProtectedMPI {
@@ -249,23 +277,6 @@ impl KeyPair {
                 })
             },
 
-            (PublicKeyAlgorithm::DSA,
-             mpi::PublicKey::DSA { p, q, g, y },
-             mpi::SecretKeyMaterial::DSA { x }) => {
-                use dsa::signature::hazmat::PrehashSigner;
-                let c = dsa::Components::from_components(
-                    p.into(), q.into(), g.into())?;
-                let public =
-                    dsa::VerifyingKey::from_components(c, y.into())?;
-                let secret =
-                    dsa::SigningKey::from_components(public, x.into())?;
-                let sig = secret.sign_prehash(digest)?;
-                Ok(mpi::Signature::DSA {
-                    r: sig.r().into(),
-                    s: sig.s().into(),
-                })
-            },
-
             (PublicKeyAlgorithm::ECDSA,
              mpi::PublicKey::ECDSA { curve, .. },
              mpi::SecretKeyMaterial::ECDSA { scalar }) => match curve
@@ -281,7 +292,7 @@ impl KeyPair {
 
                     let sig = loop {
                         let mut k: Protected = vec![0; LEN].into();
-                        crate::crypto::random(&mut k);
+                        crate::crypto::random(&mut k)?;
                         let k = Scalar::reduce_bytes(
                             GA::try_from_slice(&k)?);
                         if let Ok(s) = key.try_sign_prehashed(k, &dig) {
@@ -306,7 +317,7 @@ impl KeyPair {
 
                     let sig = loop {
                         let mut k: Protected = vec![0; LEN].into();
-                        crate::crypto::random(&mut k);
+                        crate::crypto::random(&mut k)?;
                         let k = Scalar::reduce_bytes(
                             GA::try_from_slice(&k)?);
                         if let Ok(s) = key.try_sign_prehashed(k, &dig) {
@@ -331,7 +342,7 @@ impl KeyPair {
 
                     let sig = loop {
                         let mut k: Protected = vec![0; LEN].into();
-                        crate::crypto::random(&mut k);
+                        crate::crypto::random(&mut k)?;
                         let k = Scalar::reduce_bytes(
                             GA::try_from_slice(&k)?);
                         if let Ok(s) = key.try_sign_prehashed(k, &dig) {
@@ -367,6 +378,7 @@ impl KeyPair {
                -> Result<SessionKey>
     {
         use crate::PublicKeyAlgorithm::*;
+        #[allow(deprecated)]
         match (self.public().mpis(), secret, ciphertext) {
             (mpi::PublicKey::RSA { e, n },
              mpi::SecretKeyMaterial::RSA { p, q, d, .. },
@@ -425,12 +437,13 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(), data),
 
-            RSASign | DSA | ECDSA | EdDSA =>
+            RSASign | DSA | ECDSA | EdDSA | Ed25519 | Ed448 =>
                 Err(Error::InvalidOperation(
                     format!("{} is not an encryption algorithm", self.pk_algo())
                 ).into()),
 
             ElGamalEncrypt | ElGamalEncryptSign |
+            X25519 | X448 |
             Private(_) | Unknown(_) =>
                 Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
         }
@@ -465,16 +478,7 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                 key.verify(padding, digest, &s.value_padded(key.size())?)?;
                 Ok(())
             }
-            (mpi::PublicKey::DSA { p, q, g, y },
-             mpi::Signature::DSA { r, s }) => {
-                use dsa::signature::hazmat::PrehashVerifier;
-                let c = dsa::Components::from_components(
-                    p.into(), q.into(), g.into())?;
-                let public = dsa::VerifyingKey::from_components(c, y.into())?;
-                let sig = dsa::Signature::from_components(r.into(), s.into())?;
-                public.verify_prehash(digest, &sig)?;
-                Ok(())
-            },
+
             (mpi::PublicKey::ECDSA { curve, q },
              mpi::Signature::ECDSA { r, s }) => match curve
             {
@@ -562,11 +566,11 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 impl<R> Key4<SecretParts, R>
     where R: key::KeyRole,
 {
-    /// Creates a new OpenPGP public key packet for an existing RSA key.
+    /// Creates a new OpenPGP secret key packet for an existing RSA key.
     ///
-    /// The RSA key will use public exponent `e` and modulo `n`. The key will
-    /// have it's creation date set to `ctime` or the current time if `None`
-    /// is given.
+    /// The RSA key will use the secret exponent `d`, derived from the
+    /// secret primes `p` and `q`.  The key will have its creation
+    /// date set to `ctime` or the current time if `None` is given.
     pub fn import_secret_rsa<T>(d: &[u8], p: &[u8], q: &[u8], ctime: T)
         -> Result<Self> where T: Into<Option<SystemTime>>
     {
