@@ -12,7 +12,6 @@ use crate::vec_truncate;
 use crate::{Error, Result};
 
 use crate::crypto::SessionKey;
-use crate::crypto::hash::Digest;
 use crate::crypto::mem::Protected;
 use crate::crypto::mpi::{self, MPI};
 use crate::key;
@@ -33,6 +32,7 @@ pub(crate) fn default_ecdh_kdf_hash(curve: &Curve) -> HashAlgorithm {
         Curve::NistP521 => HashAlgorithm::SHA512,
         // Extrapolated from RFC6637:
         Curve::BrainpoolP256 => HashAlgorithm::SHA256,
+        Curve::BrainpoolP384 => HashAlgorithm::SHA384,
         Curve::BrainpoolP512 => HashAlgorithm::SHA512,
         // Conservative default.
         Curve::Ed25519 // Odd: Not an encryption algo.
@@ -50,6 +50,7 @@ pub(crate) fn default_ecdh_kek_cipher(curve: &Curve) -> SymmetricAlgorithm {
         Curve::NistP521 => SymmetricAlgorithm::AES256,
         // Extrapolated from RFC6637:
         Curve::BrainpoolP256 => SymmetricAlgorithm::AES128,
+        Curve::BrainpoolP384 => SymmetricAlgorithm::AES192,
         Curve::BrainpoolP512 => SymmetricAlgorithm::AES256,
         // Conservative default.
         Curve::Ed25519 // Odd: Not an encryption algo.
@@ -68,11 +69,10 @@ pub(crate) fn default_ecdh_kek_cipher(curve: &Curve) -> SymmetricAlgorithm {
 /// (i.e. with the 0x40 prefix for X25519, or 0x04 for the NIST
 /// curves), `S` is the shared Diffie-Hellman secret.
 #[allow(non_snake_case, dead_code)]
-pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
-                              session_key: &SessionKey, VB: MPI,
-                              S: &Protected)
+pub(crate) fn encrypt_wrap(recipient: &Key<key::PublicParts, key::SubordinateRole>,
+                           session_key: &SessionKey, VB: MPI,
+                           S: &Protected)
     -> Result<mpi::Ciphertext>
-    where R: key::KeyRole
 {
     match recipient.mpis() {
         mpi::PublicKey::ECDH { ref curve, ref hash, ref sym,.. } => {
@@ -117,12 +117,12 @@ pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
 /// `recipient` is the message receiver's public key, `S` is the
 /// shared Diffie-Hellman secret used to encrypt `ciphertext`.
 #[allow(non_snake_case)]
-pub fn decrypt_unwrap2(recipient: &Key<key::PublicParts,
-                                       key::UnspecifiedRole>,
-                       S: &Protected,
-                       ciphertext: &mpi::Ciphertext,
-                       plaintext_len: Option<usize>)
-                       -> Result<SessionKey>
+pub fn decrypt_unwrap(recipient: &Key<key::PublicParts,
+                                      key::UnspecifiedRole>,
+                      S: &Protected,
+                      ciphertext: &mpi::Ciphertext,
+                      _plaintext_len: Option<usize>)
+                      -> Result<SessionKey>
 {
     match (recipient.mpis(), ciphertext) {
         (mpi::PublicKey::ECDH { ref curve, ref hash, ref sym, ..},
@@ -137,15 +137,7 @@ pub fn decrypt_unwrap2(recipient: &Key<key::PublicParts,
 
             // Compute m = AESKeyUnwrap( Z, C ) as per [RFC3394]
             let m = aes_key_unwrap(*sym, &Z, key)?;
-            let plaintext_len =
-                plaintext_len.ok_or_else(|| Error::InvalidOperation(
-                    "Need the plaintext length to decrypt this PKESK".into()))
-                .or_else(|_| -> Result<usize> {
-                    let cipher = SymmetricAlgorithm::from(m[0]);
-                    Ok(1 + cipher.key_size()? + 2)
-                })?;
-            let m = pkcs5_unpad(m, plaintext_len)?;
-
+            let m = pkcs5_unpad(m)?;
             Ok(m.into())
         },
 
@@ -155,18 +147,6 @@ pub fn decrypt_unwrap2(recipient: &Key<key::PublicParts,
     }
 }
 
-/// Unwraps a session key.
-#[allow(non_snake_case)]
-#[deprecated(note = "Use decrypt_unwrap2")]
-pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
-                         S: &Protected,
-                         ciphertext: &mpi::Ciphertext)
-    -> Result<SessionKey>
-    where R: key::KeyRole
-{
-    decrypt_unwrap2(recipient.role_as_unspecified(), S, ciphertext, None)
-}
-
 /// Derives a secret key for session key wrapping.
 ///
 /// See [Section 7 of RFC 6637].
@@ -174,7 +154,7 @@ pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
 ///   [Section 7 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-7
 fn kdf(x: &Protected, obits: usize, hash: HashAlgorithm, param: &[u8])
            -> Result<Protected> {
-    let mut hash = hash.context()?;
+    let mut hash = hash.context()?.for_digest();
     if obits > hash.digest_size() {
         return Err(
             Error::InvalidArgument("Hash digest too short".into()).into());
@@ -218,20 +198,25 @@ fn pkcs5_pad(sk: Protected, target_len: usize) -> Result<Protected> {
 /// See [Section 8 of RFC 6637].
 ///
 ///   [Section 8 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-8
-fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
+fn pkcs5_unpad(sk: Protected) -> Result<Protected> {
     if sk.len() > 0xff {
         return Err(Error::InvalidArgument("message too large".into()).into());
     }
 
-    if sk.len() < target_len {
-        return Err(Error::InvalidArgument("message too small".into()).into());
-    }
-
     let mut buf: Vec<u8> = sk.expose_into_unprotected_vec();
+
     let mut good = true;
-    let missing = (buf.len() - target_len) as u8;
-    for &b in &buf[target_len..] {
-        good = b == missing && good;
+    let mut target_len = 0;
+
+    let padding = buf[buf.len() - 1];
+    if padding == 0 || padding as usize > buf.len() {
+        // The padding can't be "0" or longer than the buffer.
+        good = false
+    } else {
+        target_len = buf.len() - padding as usize;
+        for &b in &buf[target_len..] {
+            good = b == padding && good;
+        }
     }
 
     if good {
@@ -262,7 +247,9 @@ pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    let mut cipher = algo.make_encrypt_ecb(key)?;
+    use crate::crypto::symmetric::BlockCipherMode;
+    use crate::crypto::backend::{Backend, interface::Symmetric};
+    let mut cipher = Backend::encryptor(algo, BlockCipherMode::ECB, key, None)?;
 
     //   Inputs:  Plaintext, n 64-bit values {P1, P2, ..., Pn}, and
     //            Key, K (the KEK).
@@ -297,7 +284,7 @@ pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
 
                 // A = MSB(64, B) ^ t where t = (n*j)+i
                 a = read_be_u64(&b[..8]) ^ ((n * j) + i + 1) as u64;
-                // (Note that our i runs from 0 to n-1 instead of 1 to
+                // Note that our i runs from 0 to n-1 instead of 1 to
                 // n, hence the index shift.
 
                 // R[i] = LSB(64, B)
@@ -332,11 +319,18 @@ pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    let mut cipher = algo.make_decrypt_ecb(key)?;
+    use crate::crypto::symmetric::BlockCipherMode;
+    use crate::crypto::backend::{Backend, interface::Symmetric};
+    let mut cipher = Backend::decryptor(algo, BlockCipherMode::ECB, key, None)?;
 
     //   Inputs:  Ciphertext, (n+1) 64-bit values {C0, C1, ..., Cn}, and
     //            Key, K (the KEK).
     //   Outputs: Plaintext, n 64-bit values {P1, P2, ..., Pn}.
+    if ciphertext.len() < 16 {
+        return Err(Error::InvalidArgument(
+            "Ciphertext must be at least 16 bytes".into()).into());
+    }
+
     let n = ciphertext.len() / 8 - 1;
     let mut plaintext = Vec::with_capacity(ciphertext.len() - 8);
 
@@ -363,7 +357,7 @@ pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
                 // B = AES-1(K, (A ^ t) | R[i]) where t = n*j+i
                 write_be_u64(&mut tmp[..8], a ^ ((n * j) + i + 1) as u64);
                 tmp[8..].copy_from_slice(&r[8 * i..8 * (i + 1)]);
-                // (Note that our i runs from n-1 to 0 instead of n to
+                // Note that our i runs from n-1 to 0 instead of n to
                 // 1, hence the index shift.
                 cipher.decrypt(&mut b, &tmp)?;
 
@@ -443,13 +437,20 @@ mod tests {
     fn pkcs5_padding() {
         let v = pkcs5_pad(vec![0, 0, 0].into(), 8).unwrap();
         assert_eq!(&v, &Protected::from(&[0, 0, 0, 5, 5, 5, 5, 5][..]));
-        let v = pkcs5_unpad(v, 3).unwrap();
+        let v = pkcs5_unpad(v).unwrap();
         assert_eq!(&v, &Protected::from(&[0, 0, 0][..]));
 
         let v = pkcs5_pad(vec![].into(), 8).unwrap();
         assert_eq!(&v, &Protected::from(&[8, 8, 8, 8, 8, 8, 8, 8][..]));
-        let v = pkcs5_unpad(v, 0).unwrap();
+        let v = pkcs5_unpad(v).unwrap();
         assert_eq!(&v, &Protected::from(&[][..]));
+
+        // Invalid padding.
+        let v = Protected::from(&[0, 0, 100][..]);
+        assert!(pkcs5_unpad(v).is_err());
+
+        let v = Protected::from(&[1, 0][..]);
+        assert!(pkcs5_unpad(v).is_err());
     }
 
     #[test]
@@ -594,4 +595,52 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn aes_key_unwrap_underflow() {
+        // The `aes_key_unwrap` function would panic if passed a
+        // ciphertext that was too short.  In a debug build, it would
+        // panic due to a subtraction underflow.  In a release build,
+        // it would use the small negative quantity to allocate a
+        // vector.  Since the allocator expects an unsigned quantity,
+        // the negative value would be interpreted as a huge
+        // allocation.  The allocator would then fail to allocate the
+        // memory and panic.
+        //
+        // The aes_key_unwrap function would panic if passed a
+        // ciphertext that was too short.  In a debug build, it would
+        // panic due to a subtraction underflow.  In a release build,
+        // it would use the wrapped negative quantity to allocate a
+        // vector.  Since the wrapped value is huge, it would fail to
+        // allocate the memory and panic.
+        //
+        // This test checks that short ciphertexts fail with an error
+        // and don't panic.
+
+        use crate::fmt::hex;
+
+        let key = hex::decode("c733a461b6bc6d2d15b3ac95cd02c102")
+            .expect("valid hex");
+        let key = Protected::from(key);
+
+        let ciphertext = hex::decode("\
+54a1b6d2e41fd30b34c83fc384996f7a\
+ca6904149310621e45ad14bd370a6cad\
+72d0a11048adddc856fa57e0240cd2ea")
+            .expect("valid hex");
+
+        let algo = SymmetricAlgorithm::AES128;
+
+        // Yes, this key really decryptes this cipher text.
+        assert!(aes_key_unwrap(algo.clone(), &key, &ciphertext).is_ok());
+
+        for i in 0..ciphertext.len() - 1 {
+            if let Err(err) = aes_key_unwrap(algo.clone(), &key, &ciphertext[..i]) {
+                eprintln!("{}: {}", i, err);
+            } else {
+                panic!("Expected failure for {} bytes of ciphertext, but succeeded",
+                       i);
+            }
+        }
+    }
 }
