@@ -28,18 +28,26 @@ use crate::{
     Result,
 };
 
-pub(crate) mod aead;
+pub mod aead;
 mod asymmetric;
 pub use self::asymmetric::{Signer, Decryptor, KeyPair};
 pub(crate) mod backend;
 pub mod ecdh;
 pub mod hash;
+mod key;
 pub mod mem;
 pub mod mpi;
 mod s2k;
 pub use s2k::S2K;
-pub(crate) mod symmetric;
-
+pub mod symmetric;
+mod types;
+pub use types::{
+    AEADAlgorithm,
+    Curve,
+    HashAlgorithm,
+    PublicKeyAlgorithm,
+    SymmetricAlgorithm,
+};
 #[cfg(test)]
 mod tests;
 
@@ -62,9 +70,9 @@ pub fn backend() -> String {
 /// consider using [`SessionKey::new`].
 ///
 ///   [`SessionKey::new`]: crate::crypto::SessionKey::new()
-pub fn random<B: AsMut<[u8]>>(mut buf: B) {
+pub fn random<B: AsMut<[u8]>>(mut buf: B) -> Result<()> {
     use backend::interface::Backend;
-    backend::Backend::random(buf.as_mut()).unwrap();
+    backend::Backend::random(buf.as_mut())
 }
 
 /// Holds a session key.
@@ -103,19 +111,19 @@ impl SessionKey {
     /// use openpgp::packet::prelude::*;
     ///
     /// let cipher = SymmetricAlgorithm::AES256;
-    /// let sk = SessionKey::new(cipher.key_size().unwrap());
+    /// let sk = SessionKey::new(cipher.key_size()?)?;
     ///
     /// let key: Key<key::SecretParts, key::UnspecifiedRole> =
-    ///     Key4::generate_ecc(false, Curve::Cv25519)?.into();
+    ///     Key6::generate_ecc(false, Curve::Cv25519)?.into();
     ///
     /// let pkesk: PKESK =
     ///     PKESK3::for_recipient(cipher, &sk, &key)?.into();
     /// # Ok(()) }
     /// ```
-    pub fn new(size: usize) -> Self {
+    pub fn new(size: usize) -> Result<Self> {
         let mut sk: mem::Protected = vec![0; size].into();
-        random(&mut sk);
-        Self(sk)
+        random(&mut sk)?;
+        Ok(Self(sk))
     }
 
     /// Returns a reference to the inner [`mem::Protected`].
@@ -211,13 +219,15 @@ assert_send_and_sync!(Password);
 
 impl From<Vec<u8>> for Password {
     fn from(v: Vec<u8>) -> Self {
-        Password(mem::Encrypted::new(v.into()))
+        Password(mem::Encrypted::new(v.into())
+                 .expect("encrypting memory failed"))
     }
 }
 
 impl From<Box<[u8]>> for Password {
     fn from(v: Box<[u8]>) -> Self {
-        Password(mem::Encrypted::new(v.into()))
+        Password(mem::Encrypted::new(v.into())
+                 .expect("encrypting memory failed"))
     }
 }
 
@@ -276,7 +286,7 @@ impl Password {
 /// Some encodings strip leading zero-bytes.  This function adds them
 /// back, if necessary.  If the size exceeds `to`, an error is
 /// returned.
-pub(crate) fn pad(value: &[u8], to: usize) -> Result<Cow<[u8]>>
+pub(crate) fn pad(value: &[u8], to: usize) -> Result<Cow<'_, [u8]>>
 {
     match value.len().cmp(&to) {
         Ordering::Equal => Ok(Cow::Borrowed(value)),
@@ -300,7 +310,7 @@ pub(crate) fn pad(value: &[u8], to: usize) -> Result<Cow<[u8]>>
 /// back, if necessary.  If the size exceeds `to`, the value is
 /// returned as-is.
 #[allow(dead_code)]
-pub(crate) fn pad_at_least(value: &[u8], to: usize) -> Cow<[u8]>
+pub(crate) fn pad_at_least(value: &[u8], to: usize) -> Cow<'_, [u8]>
 {
     pad(value, to).unwrap_or(Cow::Borrowed(value))
 }
@@ -311,7 +321,7 @@ pub(crate) fn pad_at_least(value: &[u8], to: usize) -> Cow<[u8]>
 /// back, if necessary.  If the size exceeds `to`, the value is
 /// silently truncated.
 #[allow(dead_code)]
-pub(crate) fn pad_truncating(value: &[u8], to: usize) -> Cow<[u8]>
+pub(crate) fn pad_truncating(value: &[u8], to: usize) -> Cow<'_, [u8]>
 {
     if value.len() == to {
         Cow::Borrowed(value)
