@@ -22,6 +22,7 @@ use std::borrow::Cow;
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
+use crate::fmt::hex;
 use crate::types::{
     Curve,
     HashAlgorithm,
@@ -524,7 +525,7 @@ impl fmt::Debug for ProtectedMPI {
 ///
 ///   [`Key`]: crate::packet::Key
 #[non_exhaustive]
-#[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
+#[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub enum PublicKey {
     /// RSA public key.
     RSA {
@@ -618,6 +619,79 @@ pub enum PublicKey {
 }
 assert_send_and_sync!(PublicKey);
 
+impl fmt::Debug for PublicKey {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            PublicKey::RSA { e, n } =>
+                f.debug_struct("RSA")
+                .field("e", e)
+                .field("n", n)
+                .finish(),
+
+            PublicKey::DSA { p, q, g, y } =>
+                f.debug_struct("DSA")
+                .field("p", p)
+                .field("q", q)
+                .field("g", g)
+                .field("y", y)
+                .finish(),
+
+            PublicKey::ElGamal { p, g, y } =>
+                f.debug_struct("ElGamal")
+                .field("p", p)
+                .field("g", g)
+                .field("y", y)
+                .finish(),
+
+            PublicKey::EdDSA { curve, q } =>
+                f.debug_struct("EdDSA")
+                .field("curve", curve)
+                .field("q", q)
+                .finish(),
+
+            PublicKey::ECDSA { curve, q } =>
+                f.debug_struct("ECDSA")
+                .field("curve", curve)
+                .field("q", q)
+                .finish(),
+
+            PublicKey::ECDH { curve, q, hash, sym } =>
+                f.debug_struct("ECDH")
+                .field("curve", curve)
+                .field("q", q)
+                .field("hash", hash)
+                .field("sym", sym)
+                .finish(),
+
+            PublicKey::X25519 { u } =>
+                f.debug_struct("X25519")
+                .field("u", &hex::encode(u))
+                .finish(),
+
+            PublicKey::X448 { u } =>
+                f.debug_struct("X448")
+                .field("u", &hex::encode(u.as_ref()))
+                .finish(),
+
+            PublicKey::Ed25519 { a } =>
+                f.debug_struct("Ed25519")
+                .field("a", &hex::encode(a))
+                .finish(),
+
+            PublicKey::Ed448 { a } =>
+                f.debug_struct("Ed448")
+                .field("a", &hex::encode(a.as_ref()))
+                .finish(),
+
+            PublicKey::Unknown { mpis, rest } =>
+                f.debug_struct("Unknown")
+                .field("mpis", mpis)
+                .field("rest", &hex::encode(rest))
+                .finish(),
+        }
+    }
+}
+
 impl PublicKey {
     /// Returns the length of the public key in bits.
     ///
@@ -832,28 +906,64 @@ impl fmt::Debug for SecretKeyMaterial {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         if cfg!(debug_assertions) {
             match self {
-                SecretKeyMaterial::RSA{ ref d, ref p, ref q, ref u } =>
-                    write!(f, "RSA {{ d: {:?}, p: {:?}, q: {:?}, u: {:?} }}", d, p, q, u),
-                SecretKeyMaterial::DSA{ ref x } =>
-                    write!(f, "DSA {{ x: {:?} }}", x),
-                SecretKeyMaterial::ElGamal{ ref x } =>
-                    write!(f, "ElGamal {{ x: {:?} }}", x),
-                SecretKeyMaterial::EdDSA{ ref scalar } =>
-                    write!(f, "EdDSA {{ scalar: {:?} }}", scalar),
-                SecretKeyMaterial::ECDSA{ ref scalar } =>
-                    write!(f, "ECDSA {{ scalar: {:?} }}", scalar),
-                SecretKeyMaterial::ECDH{ ref scalar } =>
-                    write!(f, "ECDH {{ scalar: {:?} }}", scalar),
+                SecretKeyMaterial::RSA { d, p, q, u } =>
+                    f.debug_struct("RSA")
+                    .field("d", d)
+                    .field("p", p)
+                    .field("q", q)
+                    .field("u", u)
+                    .finish(),
+
+                SecretKeyMaterial::DSA { x } =>
+                    f.debug_struct("DSA")
+                    .field("x", x)
+                    .finish(),
+
+                SecretKeyMaterial::ElGamal { x } =>
+                    f.debug_struct("ElGamal")
+                    .field("x", x)
+                    .finish(),
+
+                SecretKeyMaterial::EdDSA { scalar } =>
+                    f.debug_struct("EdDSA")
+                    .field("scalar", scalar)
+                    .finish(),
+
+                SecretKeyMaterial::ECDSA { scalar } =>
+                    f.debug_struct("ECDSA")
+                    .field("scalar", scalar)
+                    .finish(),
+
+                SecretKeyMaterial::ECDH { scalar } =>
+                    f.debug_struct("ECDH")
+                    .field("scalar", scalar)
+                    .finish(),
+
                 SecretKeyMaterial::X25519 { x } =>
-                    write!(f, "X25519 {{ x: {:?} }}", x),
+                    f.debug_struct("X25519")
+                    .field("x", &hex::encode(x))
+                    .finish(),
+
                 SecretKeyMaterial::X448 { x } =>
-                    write!(f, "X448 {{ x: {:?} }}", x),
+                    f.debug_struct("X448")
+                    .field("x", &hex::encode(x))
+                    .finish(),
+
                 SecretKeyMaterial::Ed25519 { x } =>
-                    write!(f, "Ed25519 {{ x: {:?} }}", x),
+                    f.debug_struct("Ed25519")
+                    .field("x", &hex::encode(x))
+                    .finish(),
+
                 SecretKeyMaterial::Ed448 { x } =>
-                    write!(f, "Ed448 {{ x: {:?} }}", x),
-                SecretKeyMaterial::Unknown{ ref mpis, ref rest } =>
-                    write!(f, "Unknown {{ mips: {:?}, rest: {:?} }}", mpis, rest),
+                    f.debug_struct("Ed448")
+                    .field("x", &hex::encode(x))
+                    .finish(),
+
+                SecretKeyMaterial::Unknown{ mpis, rest } =>
+                    f.debug_struct("Unknown")
+                    .field("mpis", mpis)
+                    .field("rest", &hex::encode(rest))
+                    .finish(),
             }
         } else {
             match self {
@@ -1112,7 +1222,7 @@ impl SecretKeyChecksum {
 ///
 ///   [`PKESK`]: crate::packet::PKESK
 #[non_exhaustive]
-#[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
+#[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub enum Ciphertext {
     /// RSA ciphertext.
     RSA {
@@ -1162,6 +1272,47 @@ pub enum Ciphertext {
 }
 assert_send_and_sync!(Ciphertext);
 
+impl fmt::Debug for Ciphertext {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Ciphertext::RSA { c } =>
+                f.debug_struct("RSA")
+                .field("c", c)
+                .finish(),
+
+            Ciphertext::ElGamal { e, c } =>
+                f.debug_struct("ElGamal")
+                .field("e", e)
+                .field("c", c)
+                .finish(),
+
+            Ciphertext::ECDH { e, key } =>
+                f.debug_struct("ECDH")
+                .field("e", e)
+                .field("key", &hex::encode(key))
+                .finish(),
+
+            Ciphertext::X25519 { e, key } =>
+                f.debug_struct("X25519")
+                .field("e", &hex::encode(&e[..]))
+                .field("key", &hex::encode(key))
+                .finish(),
+
+            Ciphertext::X448 { e, key } =>
+                f.debug_struct("X448")
+                .field("e", &hex::encode(&e[..]))
+                .field("key", &hex::encode(key))
+                .finish(),
+
+            Ciphertext::Unknown { mpis, rest } =>
+                f.debug_struct("Unknown")
+                .field("mpis", mpis)
+                .field("rest", &hex::encode(rest))
+                .finish(),
+        }
+    }
+}
+
 impl Ciphertext {
     /// Returns, if known, the public-key algorithm for this
     /// ciphertext.
@@ -1242,7 +1393,7 @@ impl Arbitrary for Ciphertext {
 ///
 ///   [`Signature`]: crate::packet::Signature
 #[non_exhaustive]
-#[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
+#[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub enum Signature {
     /// RSA signature.
     RSA {
@@ -1304,6 +1455,57 @@ pub enum Signature {
 }
 assert_send_and_sync!(Signature);
 
+impl fmt::Debug for Signature {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Signature::RSA { s } =>
+                f.debug_struct("RSA")
+                .field("s", s)
+                .finish(),
+
+            Signature::DSA { r, s } =>
+                f.debug_struct("DSA")
+                .field("r", r)
+                .field("s", s)
+                .finish(),
+
+            Signature::ElGamal { r, s } =>
+                f.debug_struct("ElGamal")
+                .field("r", r)
+                .field("s", s)
+                .finish(),
+
+            Signature::EdDSA { r, s } =>
+                f.debug_struct("EdDSA")
+                .field("r", r)
+                .field("s", s)
+                .finish(),
+
+            Signature::ECDSA { r, s } =>
+                f.debug_struct("ECDSA")
+                .field("r", r)
+                .field("s", s)
+                .finish(),
+
+            Signature::Ed25519 { s } =>
+                f.debug_struct("Ed25519")
+                .field("s", &hex::encode(&s[..]))
+                .finish(),
+
+            Signature::Ed448 { s } =>
+                f.debug_struct("Ed448")
+                .field("s", &hex::encode(&s[..]))
+                .finish(),
+
+            Signature::Unknown { mpis, rest } =>
+                f.debug_struct("Unknown")
+                .field("mpis", mpis)
+                .field("rest", &hex::encode(rest))
+                .finish(),
+        }
+    }
+}
+
 impl Hash for Signature {
     fn hash(&self, mut hash: &mut hash::Context) -> Result<()> {
         self.serialize(&mut hash as &mut dyn Write)
@@ -1364,37 +1566,11 @@ mod tests {
     quickcheck! {
         fn pk_roundtrip(pk: PublicKey) -> bool {
             use std::io::Cursor;
-            use crate::PublicKeyAlgorithm::*;
 
             let mut buf = Vec::new();
             pk.serialize(&mut buf).unwrap();
             let cur = Cursor::new(buf);
-
-            #[allow(deprecated)]
-            let pk_ = match &pk {
-                PublicKey::RSA { .. } =>
-                    PublicKey::parse(RSAEncryptSign, cur).unwrap(),
-                PublicKey::DSA { .. } =>
-                    PublicKey::parse(DSA, cur).unwrap(),
-                PublicKey::ElGamal { .. } =>
-                    PublicKey::parse(ElGamalEncrypt, cur).unwrap(),
-                PublicKey::EdDSA { .. } =>
-                    PublicKey::parse(EdDSA, cur).unwrap(),
-                PublicKey::ECDSA { .. } =>
-                    PublicKey::parse(ECDSA, cur).unwrap(),
-                PublicKey::ECDH { .. } =>
-                    PublicKey::parse(ECDH, cur).unwrap(),
-                PublicKey::X25519 { .. } =>
-                    PublicKey::parse(X25519, cur).unwrap(),
-                PublicKey::X448 { .. } =>
-                    PublicKey::parse(X448, cur).unwrap(),
-                PublicKey::Ed25519 { .. } =>
-                    PublicKey::parse(Ed25519, cur).unwrap(),
-                PublicKey::Ed448 { .. } =>
-                    PublicKey::parse(Ed448, cur).unwrap(),
-
-                PublicKey::Unknown { .. } => unreachable!(),
-            };
+            let pk_ = PublicKey::parse(pk.algo().unwrap(), cur).unwrap();
 
             pk == pk_
         }
@@ -1421,38 +1597,11 @@ mod tests {
 
     quickcheck! {
         fn sk_roundtrip(sk: SecretKeyMaterial) -> bool {
-            use std::io::Cursor;
-            use crate::PublicKeyAlgorithm::*;
-
             let mut buf = Vec::new();
             sk.serialize(&mut buf).unwrap();
-            let cur = Cursor::new(buf);
-
-            #[allow(deprecated)]
-            let sk_ = match &sk {
-                SecretKeyMaterial::RSA { .. } =>
-                    SecretKeyMaterial::parse(RSAEncryptSign, cur).unwrap(),
-                SecretKeyMaterial::DSA { .. } =>
-                    SecretKeyMaterial::parse(DSA, cur).unwrap(),
-                SecretKeyMaterial::EdDSA { .. } =>
-                    SecretKeyMaterial::parse(EdDSA, cur).unwrap(),
-                SecretKeyMaterial::ECDSA { .. } =>
-                    SecretKeyMaterial::parse(ECDSA, cur).unwrap(),
-                SecretKeyMaterial::ECDH { .. } =>
-                    SecretKeyMaterial::parse(ECDH, cur).unwrap(),
-                SecretKeyMaterial::ElGamal { .. } =>
-                    SecretKeyMaterial::parse(ElGamalEncrypt, cur).unwrap(),
-                SecretKeyMaterial::X25519 { .. } =>
-                    SecretKeyMaterial::parse(X25519, cur).unwrap(),
-                SecretKeyMaterial::X448 { .. } =>
-                    SecretKeyMaterial::parse(X448, cur).unwrap(),
-                SecretKeyMaterial::Ed25519 { .. } =>
-                    SecretKeyMaterial::parse(Ed25519, cur).unwrap(),
-                SecretKeyMaterial::Ed448 { .. } =>
-                    SecretKeyMaterial::parse(Ed448, cur).unwrap(),
-
-                SecretKeyMaterial::Unknown { .. } => unreachable!(),
-            };
+            let sk_ =
+                SecretKeyMaterial::from_bytes(sk.algo().unwrap(),
+                                              &buf).unwrap();
 
             sk == sk_
         }
@@ -1461,27 +1610,11 @@ mod tests {
     quickcheck! {
         fn ct_roundtrip(ct: Ciphertext) -> bool {
             use std::io::Cursor;
-            use crate::PublicKeyAlgorithm::*;
 
             let mut buf = Vec::new();
             ct.serialize(&mut buf).unwrap();
             let cur = Cursor::new(buf);
-
-            #[allow(deprecated)]
-            let ct_ = match &ct {
-                Ciphertext::RSA { .. } =>
-                    Ciphertext::parse(RSAEncryptSign, cur).unwrap(),
-                Ciphertext::ElGamal { .. } =>
-                    Ciphertext::parse(ElGamalEncrypt, cur).unwrap(),
-                Ciphertext::ECDH { .. } =>
-                    Ciphertext::parse(ECDH, cur).unwrap(),
-                Ciphertext::X25519 { .. } =>
-                    Ciphertext::parse(X25519, cur).unwrap(),
-                Ciphertext::X448 { .. } =>
-                    Ciphertext::parse(X448, cur).unwrap(),
-
-                Ciphertext::Unknown { .. } => unreachable!(),
-            };
+            let ct_ = Ciphertext::parse(ct.pk_algo().unwrap(), cur).unwrap();
 
             ct == ct_
         }
