@@ -21,7 +21,6 @@
 //! ```
 
 pub use crate::packet::{
-    AED,
     Any,
     Body,
     CompressedData,
@@ -34,6 +33,7 @@ pub use crate::packet::{
     OnePassSig,
     PKESK,
     Packet,
+    Padding,
     SEIP,
     SKESK,
     Signature,
@@ -42,17 +42,25 @@ pub use crate::packet::{
     Unknown,
     UserAttribute,
     UserID,
-    aed::AED1,
     key,
     key::Key4,
+    key::Key6,
     key::SecretKeyMaterial,
     one_pass_sig::OnePassSig3,
-    pkesk::PKESK3,
-    seip::SEIP1,
+    one_pass_sig::OnePassSig6,
+    pkesk::{
+        PKESK3,
+        PKESK6,
+    },
+    seip::{
+        SEIP1,
+        SEIP2,
+    },
     signature,
     signature::Signature4,
+    signature::Signature6,
     signature::SignatureBuilder,
     skesk::SKESK4,
-    skesk::SKESK5,
+    skesk::SKESK6,
     user_attribute,
 };
