@@ -10,7 +10,10 @@ use crate::{
     HashAlgorithm,
 };
 use crate::types::Curve;
-use crate::crypto::mpi::{self, MPI, ProtectedMPI};
+use crate::crypto::{
+    mem::Protected,
+    mpi::{self, MPI, ProtectedMPI},
+};
 use crate::parse::{
     PacketHeaderParser,
     Cookie,
@@ -19,9 +22,9 @@ use crate::parse::{
 impl mpi::PublicKey {
     /// Parses a set of OpenPGP MPIs representing a public key.
     ///
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub fn parse<R: Read + Send + Sync>(algo: PublicKeyAlgorithm, reader: R) -> Result<Self>
     {
         let bio = buffered_reader::Generic::with_cookie(
@@ -32,9 +35,9 @@ impl mpi::PublicKey {
 
     /// Parses a set of OpenPGP MPIs representing a public key.
     ///
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
         php: &mut PacketHeaderParser<'_>)
@@ -130,6 +133,30 @@ impl mpi::PublicKey {
                 })
             }
 
+            X25519 => {
+                let mut u = [0; 32];
+                php.parse_bytes_into("x25519_public", &mut u)?;
+                Ok(mpi::PublicKey::X25519 { u })
+            },
+
+            X448 => {
+                let mut u = [0; 56];
+                php.parse_bytes_into("x448_public", &mut u)?;
+                Ok(mpi::PublicKey::X448 { u: Box::new(u) })
+            },
+
+            Ed25519 => {
+                let mut a = [0; 32];
+                php.parse_bytes_into("ed25519_public", &mut a)?;
+                Ok(mpi::PublicKey::Ed25519 { a })
+            },
+
+            Ed448 => {
+                let mut a = [0; 57];
+                php.parse_bytes_into("ed448_public", &mut a)?;
+                Ok(mpi::PublicKey::Ed448 { a: Box::new(a) })
+            },
+
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
                 while let Ok(mpi) = MPI::parse("unknown_len",
@@ -179,9 +206,9 @@ impl mpi::SecretKeyMaterial {
 
     /// Parses a set of OpenPGP MPIs representing a secret key.
     ///
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     #[deprecated(
         since = "1.14.0",
         note = "Leaks secrets into the heap, use [`SecretKeyMaterial::from_bytes`]")]
@@ -195,9 +222,9 @@ impl mpi::SecretKeyMaterial {
 
     /// Parses a set of OpenPGP MPIs representing a secret key.
     ///
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub fn from_bytes(algo: PublicKeyAlgorithm, buf: &[u8]) -> Result<Self> {
         let bio = buffered_reader::Memory::with_cookie(
             buf, Cookie::default());
@@ -207,9 +234,9 @@ impl mpi::SecretKeyMaterial {
 
     /// Parses a set of OpenPGP MPIs representing a secret key.
     ///
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
         php: &mut PacketHeaderParser<'_>,
@@ -274,6 +301,30 @@ impl mpi::SecretKeyMaterial {
                 })
             }
 
+            X25519 => {
+                let mut x: Protected = vec![0; 32].into();
+                php.parse_bytes_into("x25519_secret", &mut x)?;
+                Ok(mpi::SecretKeyMaterial::X25519 { x })
+            },
+
+            X448 => {
+                let mut x: Protected = vec![0; 56].into();
+                php.parse_bytes_into("x448_secret", &mut x)?;
+                Ok(mpi::SecretKeyMaterial::X448 { x })
+            },
+
+            Ed25519 => {
+                let mut x: Protected = vec![0; 32].into();
+                php.parse_bytes_into("ed25519_secret", &mut x)?;
+                Ok(mpi::SecretKeyMaterial::Ed25519 { x })
+            },
+
+            Ed448 => {
+                let mut x: Protected = vec![0; 57].into();
+                php.parse_bytes_into("ed448_secret", &mut x)?;
+                Ok(mpi::SecretKeyMaterial::Ed448 { x })
+            },
+
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
                 while let Ok(mpi) = ProtectedMPI::parse("unknown_len",
@@ -314,7 +365,8 @@ impl mpi::SecretKeyMaterial {
             let good = match checksum {
                 mpi::SecretKeyChecksum::SHA1 => {
                     // Compute SHA1 hash.
-                    let mut hsh = HashAlgorithm::SHA1.context().unwrap();
+                    let mut hsh = HashAlgorithm::SHA1.context().unwrap()
+                        .for_digest();
                     hsh.update(data);
                     let mut our_chksum = [0u8; 20];
                     let _ = hsh.digest(&mut our_chksum);
@@ -354,9 +406,9 @@ impl mpi::Ciphertext {
     /// Parses a set of OpenPGP MPIs representing a ciphertext.
     ///
     /// Expects MPIs for a public key algorithm `algo`s ciphertext.
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub fn parse<R: Read + Send + Sync>(algo: PublicKeyAlgorithm, reader: R) -> Result<Self>
     {
         let bio = buffered_reader::Generic::with_cookie(
@@ -368,9 +420,9 @@ impl mpi::Ciphertext {
     /// Parses a set of OpenPGP MPIs representing a ciphertext.
     ///
     /// Expects MPIs for a public key algorithm `algo`s ciphertext.
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
         php: &mut PacketHeaderParser<'_>)
@@ -409,6 +461,24 @@ impl mpi::Ciphertext {
                 })
             }
 
+            X25519 => {
+                let mut e = [0; 32];
+                php.parse_bytes_into("x25519_e", &mut e)?;
+                let key_len = php.parse_u8("x25519_esk_len")? as usize;
+                let key = Vec::from(&php.parse_bytes("x25519_esk", key_len)?
+                                    [..key_len]);
+                Ok(mpi::Ciphertext::X25519 { e: Box::new(e), key: key.into() })
+            },
+
+            X448 => {
+                let mut e = [0; 56];
+                php.parse_bytes_into("x448_e", &mut e)?;
+                let key_len = php.parse_u8("x448_esk_len")? as usize;
+                let key = Vec::from(&php.parse_bytes("x448_esk", key_len)?
+                                    [..key_len]);
+                Ok(mpi::Ciphertext::X448 { e: Box::new(e), key: key.into() })
+            },
+
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
                 while let Ok(mpi) = MPI::parse("unknown_len",
@@ -423,8 +493,9 @@ impl mpi::Ciphertext {
                 })
             }
 
-            RSASign | DSA | EdDSA | ECDSA => Err(Error::InvalidArgument(
-                format!("not an encryption algorithm: {:?}", algo)).into()),
+            RSASign | DSA | EdDSA | ECDSA | Ed25519 | Ed448
+                => Err(Error::InvalidArgument(
+                    format!("not an encryption algorithm: {:?}", algo)).into()),
         }
     }
 }
@@ -433,9 +504,9 @@ impl mpi::Signature {
     /// Parses a set of OpenPGP MPIs representing a signature.
     ///
     /// Expects MPIs for a public key algorithm `algo`s signature.
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub fn parse<R: Read + Send + Sync>(algo: PublicKeyAlgorithm, reader: R) -> Result<Self>
     {
         let bio = buffered_reader::Generic::with_cookie(
@@ -447,9 +518,9 @@ impl mpi::Signature {
     /// Parses a set of OpenPGP MPIs representing a signature.
     ///
     /// Expects MPIs for a public key algorithm `algo`s signature.
-    /// See [Section 3.2 of RFC 4880] for details.
+    /// See [Section 3.2 of RFC 9580] for details.
     ///
-    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    ///   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
     pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
         php: &mut PacketHeaderParser<'_>)
@@ -514,6 +585,18 @@ impl mpi::Signature {
                 })
             }
 
+            Ed25519 => {
+                let mut s = [0; 64];
+                php.parse_bytes_into("ed25519_sig", &mut s)?;
+                Ok(mpi::Signature::Ed25519 { s: Box::new(s) })
+            },
+
+            Ed448 => {
+                let mut s = [0; 114];
+                php.parse_bytes_into("ed448_sig", &mut s)?;
+                Ok(mpi::Signature::Ed448 { s: Box::new(s) })
+            },
+
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
                 while let Ok(mpi) = MPI::parse("unknown_len",
@@ -528,8 +611,9 @@ impl mpi::Signature {
                 })
             }
 
-            RSAEncrypt | ElGamalEncrypt | ECDH => Err(Error::InvalidArgument(
-                format!("not a signature algorithm: {:?}", algo)).into()),
+            RSAEncrypt | ElGamalEncrypt | ECDH | X25519 | X448
+                => Err(Error::InvalidArgument(
+                    format!("not a signature algorithm: {:?}", algo)).into()),
         }
     }
 }
