@@ -1,7 +1,4 @@
 use std::convert::TryFrom;
-use std::io;
-use std::ops::{Deref, DerefMut};
-use std::path::Path;
 
 use crate::{
     Result,
@@ -10,13 +7,14 @@ use crate::{
 };
 use crate::parse::{
     PacketParserBuilder,
+    PacketParserEOF,
     PacketParserResult,
     PacketParser,
     Parse,
     Cookie
 };
 use buffered_reader::BufferedReader;
-
+
 /// Parses an OpenPGP stream with the convenience of
 /// [`PacketPile::from_file`] and the flexibility of a
 /// [`PacketParser`].
@@ -63,7 +61,7 @@ use buffered_reader::BufferedReader;
 /// let mut ppp =
 ///     PacketPileParser::from_bytes(
 ///         b"\xcb\x12b\x00\x00\x00\x00\x00Hello world.")?;
-/// while ppp.is_some() {
+/// while ppp.packet().is_ok() {
 ///     // Start parsing the next packet, recursing.
 ///     ppp.recurse()?;
 /// }
@@ -97,7 +95,7 @@ use buffered_reader::BufferedReader;
 ///         b"\xcb\x12b\x00\x00\x00\x00\x00Hello world.")?
 ///     .buffer_unread_content();
 /// let mut ppp = PacketPileParser::try_from(ppb)?;
-/// while ppp.is_some() {
+/// while ppp.packet().is_ok() {
 ///     // Start parsing the next packet, recursing.
 ///     ppp.recurse()?;
 /// }
@@ -126,7 +124,7 @@ use buffered_reader::BufferedReader;
 /// let mut ppp =
 ///     PacketPileParser::from_bytes(
 ///         b"\xcb\x12b\x00\x00\x00\x00\x00Hello world.")?;
-/// while let Ok(pp) = ppp.as_mut() {
+/// while let Ok(pp) = ppp.packet_mut() {
 ///     if let Packet::Literal(_) = pp.packet {
 ///         // Buffer this packet's body.
 ///         pp.buffer_unread_content()?;
@@ -161,7 +159,7 @@ use buffered_reader::BufferedReader;
 /// let mut ppp =
 ///     PacketPileParser::from_bytes(
 ///         b"\xcb\x12b\x00\x00\x00\x00\x00Hello world.")?;
-/// while let Ok(pp) = ppp.as_mut() {
+/// while let Ok(pp) = ppp.packet_mut() {
 ///     if let Packet::Literal(_) = pp.packet {
 ///         // Stream the body.
 ///         let mut buf = Vec::new();
@@ -193,20 +191,6 @@ pub struct PacketPileParser<'a> {
 }
 assert_send_and_sync!(PacketPileParser<'_>);
 
-impl<'a> Deref for PacketPileParser<'a> {
-    type Target = PacketParserResult<'a>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.ppr
-    }
-}
-
-impl<'a> DerefMut for PacketPileParser<'a> {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.ppr
-    }
-}
-
 impl<'a> TryFrom<PacketParserBuilder<'a>> for PacketPileParser<'a> {
     type Error = anyhow::Error;
 
@@ -224,34 +208,10 @@ impl<'a> Parse<'a, PacketPileParser<'a>> for PacketPileParser<'a> {
     {
         PacketPileParser::from_cookie_reader(reader.into_boxed())
     }
-
-    /// Creates a `PacketPileParser` to parse the OpenPGP message stored
-    /// in the `io::Read` object.
-    fn from_reader<R: io::Read + 'a + Send + Sync>(reader: R)
-             -> Result<PacketPileParser<'a>> {
-        let bio = Box::new(buffered_reader::Generic::with_cookie(
-            reader, None, Cookie::default()));
-        PacketPileParser::from_cookie_reader(bio)
-    }
-
-    /// Creates a `PacketPileParser` to parse the OpenPGP message stored
-    /// in the file named by `path`.
-    fn from_file<P: AsRef<Path>>(path: P)
-            -> Result<PacketPileParser<'a>> {
-        PacketPileParser::from_cookie_reader(
-            Box::new(buffered_reader::File::with_cookie(path, Cookie::default())?))
-    }
-
-    /// Creates a `PacketPileParser` to parse the OpenPGP message stored
-    /// in the provided buffer.
-    fn from_bytes<D: AsRef<[u8]> + ?Sized + Send + Sync>(data: &'a D)
-            -> Result<PacketPileParser<'a>> {
-        let bio = Box::new(buffered_reader::Memory::with_cookie(
-            data.as_ref(), Cookie::default()));
-        PacketPileParser::from_cookie_reader(bio)
-    }
 }
 
+impl<'a> crate::seal::Sealed for PacketPileParser<'a> {}
+
 impl<'a> PacketPileParser<'a> {
     /// Creates a `PacketPileParser` from a *fresh* `PacketParser`.
     fn from_packet_parser(ppr: PacketParserResult<'a>)
@@ -295,6 +255,21 @@ impl<'a> PacketPileParser<'a> {
         container.children_mut().unwrap().push(packet);
     }
 
+    /// Returns a reference to the current packet.
+    pub fn packet(&self)
+                  -> std::result::Result<&PacketParser<'a>, &PacketParserEOF>
+    {
+        self.ppr.as_ref()
+    }
+
+    /// Returns a mutable reference to the current packet.
+    pub fn packet_mut<>(&mut self)
+                        -> std::result::Result<&mut PacketParser<'a>,
+                                               &mut PacketParserEOF<'a>>
+    {
+        self.ppr.as_mut()
+    }
+
     /// Finishes parsing the current packet and starts parsing the
     /// next one, recursing if possible.
     ///
@@ -322,7 +297,7 @@ impl<'a> PacketPileParser<'a> {
     /// let message_data: &[u8] = // ...
     /// #    include_bytes!("../../tests/data/messages/compressed-data-algo-0.pgp");
     /// let mut ppp = PacketPileParser::from_bytes(message_data)?;
-    /// while let Ok(pp) = ppp.as_ref() {
+    /// while let Ok(pp) = ppp.packet() {
     ///     // Do something interesting with `pp` here.
     ///
     ///     // Start parsing the next packet, recursing.
@@ -375,7 +350,7 @@ impl<'a> PacketPileParser<'a> {
     /// let message_data: &[u8] = // ...
     /// #    include_bytes!("../../tests/data/messages/compressed-data-algo-0.pgp");
     /// let mut ppp = PacketPileParser::from_bytes(message_data)?;
-    /// while let Ok(pp) = ppp.as_ref() {
+    /// while let Ok(pp) = ppp.packet() {
     ///     // Do something interesting with `pp` here.
     ///
     ///     // Start parsing the next packet.
@@ -420,7 +395,7 @@ impl<'a> PacketPileParser<'a> {
     /// let message_data: &[u8] = // ...
     /// #    include_bytes!("../../tests/data/messages/compressed-data-algo-0.pgp");
     /// let mut ppp = PacketPileParser::from_bytes(message_data)?;
-    /// while let Ok(pp) = ppp.as_ref() {
+    /// while let Ok(pp) = ppp.packet() {
     ///     match pp.packet {
     ///         Packet::CompressedData(_) =>
     ///             assert_eq!(ppp.recursion_depth(), Some(0)),
@@ -462,7 +437,7 @@ impl<'a> PacketPileParser<'a> {
     /// let message_data: &[u8] = // ...
     /// #    include_bytes!("../../tests/data/messages/compressed-data-algo-0.pgp");
     /// let mut ppp = PacketPileParser::from_bytes(message_data)?;
-    /// while ppp.is_some() {
+    /// while ppp.packet().is_ok() {
     ///     // Start parsing the next packet.
     ///     ppp.next()?;
     /// }
@@ -512,8 +487,8 @@ impl<'a> PacketPileParser<'a> {
 fn test_recurse() -> Result<()> {
     let mut count = 0;
     let mut ppp =
-        PacketPileParser::from_bytes(crate::tests::key("public-key.gpg"))?;
-    while ppp.is_some() {
+        PacketPileParser::from_bytes(crate::tests::key("public-key.pgp"))?;
+    while ppp.packet().is_ok() {
         count += 1;
         ppp.recurse().unwrap();
     }
@@ -527,8 +502,8 @@ fn test_recurse() -> Result<()> {
 fn test_next() -> Result<()> {
     let mut count = 0;
     let mut ppp =
-        PacketPileParser::from_bytes(crate::tests::key("public-key.gpg"))?;
-    while ppp.is_some() {
+        PacketPileParser::from_bytes(crate::tests::key("public-key.pgp"))?;
+    while ppp.packet().is_ok() {
         count += 1;
         ppp.next().unwrap();
     }
@@ -550,9 +525,9 @@ fn message_parser_reader_interface() {
     // A message containing a compressed packet that contains a
     // literal packet.
     let mut ppp = PacketPileParser::from_bytes(
-        crate::tests::message("compressed-data-algo-1.gpg")).unwrap();
+        crate::tests::message("compressed-data-algo-1.pgp")).unwrap();
     let mut count = 0;
-    while let Ok(pp) = ppp.as_mut() {
+    while let Ok(pp) = ppp.packet_mut() {
         if let Packet::Literal(_) = pp.packet {
             assert_eq!(count, 1); // The *second* packet.
 
